@@ -328,7 +328,7 @@ static unsigned int parse_directory(struct entry *root_entry, const char *name,
 		memcpy(endpath, dirent->d_name, namelen + 1);
 
 		if (lstat(path, &st) < 0) {
-			perror(endpath);
+			warn(_("stat of %s failed"), endpath);
 			warn_skip = 1;
 			continue;
 		}
@@ -748,7 +748,7 @@ int main(int argc, char **argv)
 		case 'i':
 			opt_image = optarg;
 			if (lstat(opt_image, &st) < 0)
-				err(MKFS_EX_USAGE, _("stat failed %s"), opt_image);
+				err(MKFS_EX_USAGE, _("stat of %s failed"), opt_image);
 			image_length = st.st_size; /* may be padded later */
 			fslen_ub += (image_length + 3); /* 3 is for padding */
 			break;
@@ -785,7 +785,7 @@ int main(int argc, char **argv)
 		blksize = getpagesize();
 
 	if (stat(dirname, &st) < 0)
-		err(MKFS_EX_USAGE, _("stat failed %s"), dirname);
+		err(MKFS_EX_USAGE, _("stat of %s failed"), dirname);
 	fd = open(outfile, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 	if (fd < 0)
 		err(MKFS_EX_USAGE, _("cannot open %s"), outfile);
