@@ -54,6 +54,10 @@ static void __attribute__ ((__noreturn__)) usage(int err)
 		  " %1$s -q %2$srawN\n"
 		  " %1$s -qa\n"), program_invocation_short_name,
 		_PATH_RAWDEVDIR);
+
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("Bind a raw character device to a block device.\n"), out);
+
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -q, --query    set query mode\n"), out);
 	fputs(_(" -a, --all      query all raw devices\n"), out);
@@ -104,7 +108,7 @@ int main(int argc, char *argv[])
 		{NULL, no_argument, 0, '0'},
 	};
 
-	setlocale(LC_MESSAGES, "");
+	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	atexit(close_stdout);
@@ -220,7 +224,7 @@ static int query(int minor_raw, const char *raw_name, int quiet)
 	if (raw_name) {
 		struct stat statbuf;
 
-		if (!stat(raw_name, &statbuf))
+		if (stat(raw_name, &statbuf) != 0)
 			err(EXIT_RAW_ACCESS,
 			    _("Cannot locate raw device '%s'"), raw_name);
 		if (!S_ISCHR(statbuf.st_mode))
@@ -261,7 +265,7 @@ static int bind(int minor_raw, int block_major, int block_minor)
 	rq.raw_minor = minor_raw;
 	rq.block_major = block_major;
 	rq.block_minor = block_minor;
-	if (!ioctl(master_fd, RAW_SETBIND, &rq))
+	if (ioctl(master_fd, RAW_SETBIND, &rq) < 0)
 		err(EXIT_RAW_IOCTL, _("Error setting raw device"));
 	printf(_("%sraw%d:  bound to major %d, minor %d\n"),
 	       _PATH_RAWDEVDIR, raw_minor, (int)rq.block_major,
