@@ -7,6 +7,9 @@
  *
  * Based on code from taskset.c and Linux kernel.
  *
+ * This file may be redistributed under the terms of the
+ * GNU Lesser General Public License.
+ *
  * Copyright (C) 2010 Karel Zak <kzak@redhat.com>
  */
 
@@ -385,6 +388,7 @@ int main(int argc, char *argv[])
 	printf("[%s]\n", cpulist_create(buf, buflen, set, setsize));
 
 	free(buf);
+	free(mask);
 	free(range);
 	cpuset_free(set);
 
