@@ -1,40 +1,102 @@
 
 #include "strutils.h"
-
 #include "fdiskP.h"
 
+/**
+ * SECTION: ask
+ * @title: Ask
+ * @short_description: interface for dialog driven partitioning, warning and info messages
+ *
+ */
+
 static void fdisk_ask_menu_reset_items(struct fdisk_ask *ask);
 
 
+/**
+ * fdisk_set_ask:
+ * @cxt: context
+ * @ask_cb: callback
+ * @data: callback data
+ *
+ * Set callback for dialog driven partitioning and library warnings/errors.
+ *
+ * Returns: 0 on success, < 0 on error.
+ */
+int fdisk_set_ask(struct fdisk_context *cxt,
+		int (*ask_cb)(struct fdisk_context *, struct fdisk_ask *, void *),
+		void *data)
+{
+	assert(cxt);
+
+	cxt->ask_cb = ask_cb;
+	cxt->ask_data = data;
+	return 0;
+}
+
 struct fdisk_ask *fdisk_new_ask(void)
 {
 	struct fdisk_ask *ask = calloc(1, sizeof(struct fdisk_ask));
 	DBG(ASK, ul_debugobj(ask, "alloc"));
+	ask->refcount = 1;
 	return ask;
 }
 
 void fdisk_reset_ask(struct fdisk_ask *ask)
 {
+	int refcount;
+
 	assert(ask);
 	free(ask->query);
 
 	DBG(ASK, ul_debugobj(ask, "reset"));
+	refcount = ask->refcount;
 
 	if (fdisk_is_ask(ask, MENU))
 		fdisk_ask_menu_reset_items(ask);
 
 	memset(ask, 0, sizeof(*ask));
+	ask->refcount = refcount;
 }
 
-void fdisk_free_ask(struct fdisk_ask *ask)
+/**
+ * fdisk_ref_ask:
+ * @ask: ask instance
+ *
+ * Incremparts reference counter.
+ */
+void fdisk_ref_ask(struct fdisk_ask *ask)
+{
+	if (ask)
+		ask->refcount++;
+}
+
+
+/**
+ * fdisk_unref_ask:
+ * @ask: ask instance
+ *
+ * De-incremparts reference counter, on zero the @ask is automatically
+ * deallocated.
+ */
+void fdisk_unref_ask(struct fdisk_ask *ask)
 {
 	if (!ask)
 		return;
-	fdisk_reset_ask(ask);
-	DBG(ASK, ul_debugobj(ask, "free"));
-	free(ask);
+	ask->refcount--;
+
+	if (ask->refcount <= 0) {
+		fdisk_reset_ask(ask);
+		DBG(ASK, ul_debugobj(ask, "free"));
+		free(ask);
+	}
 }
 
+/**
+ * fdisk_ask_get_query:
+ * @ask: ask instance
+ *
+ * Returns: pointer to dialog string.
+ */
 const char *fdisk_ask_get_query(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -47,6 +109,12 @@ int fdisk_ask_set_query(struct fdisk_ask *ask, const char *str)
 	return !strdup_to_struct_member(ask, query, str) ? -ENOMEM : 0;
 }
 
+/**
+ * fdisk_ask_get_type:
+ * @ask: ask instance
+ *
+ * Returns: FDISK_ASKTYPE_*
+ */
 int fdisk_ask_get_type(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -60,19 +128,6 @@ int fdisk_ask_set_type(struct fdisk_ask *ask, int type)
 	return 0;
 }
 
-unsigned int fdisk_ask_get_flags(struct fdisk_ask *ask)
-{
-	assert(ask);
-	return ask->flags;
-}
-
-int fdisk_ask_set_flags(struct fdisk_ask *ask, unsigned int flags)
-{
-	assert(ask);
-	ask->flags = flags;
-	return 0;
-}
-
 int fdisk_do_ask(struct fdisk_context *cxt, struct fdisk_ask *ask)
 {
 	int rc;
@@ -100,6 +155,12 @@ int fdisk_do_ask(struct fdisk_context *cxt, struct fdisk_ask *ask)
 
 #define is_number_ask(a)  (fdisk_is_ask(a, NUMBER) || fdisk_is_ask(a, OFFSET))
 
+/**
+ * fdisk_ask_number_get_range:
+ * @ask: ask instance
+ *
+ * Returns: string with range (e.g. "1,3,5-10")
+ */
 const char *fdisk_ask_number_get_range(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -115,6 +176,13 @@ int fdisk_ask_number_set_range(struct fdisk_ask *ask, const char *range)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_get_default:
+ * @ask: ask instance
+ *
+ * Returns: default number
+ *
+ */
 uint64_t fdisk_ask_number_get_default(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -129,6 +197,12 @@ int fdisk_ask_number_set_default(struct fdisk_ask *ask, uint64_t dflt)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_get_low:
+ * @ask: ask instance
+ *
+ * Returns: minimal possible number when ask for numbers in range
+ */
 uint64_t fdisk_ask_number_get_low(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -143,6 +217,12 @@ int fdisk_ask_number_set_low(struct fdisk_ask *ask, uint64_t low)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_get_high:
+ * @ask: ask instance
+ *
+ * Returns: maximal possible number when ask for numbers in range
+ */
 uint64_t fdisk_ask_number_get_high(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -157,6 +237,12 @@ int fdisk_ask_number_set_high(struct fdisk_ask *ask, uint64_t high)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_get_result:
+ * @ask: ask instance
+ *
+ * Returns: result
+ */
 uint64_t fdisk_ask_number_get_result(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -164,6 +250,13 @@ uint64_t fdisk_ask_number_get_result(struct fdisk_ask *ask)
 	return ask->data.num.result;
 }
 
+/**
+ * fdisk_ask_number_set_result:
+ * @ask: ask instance
+ * @result: dialog result
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_ask_number_set_result(struct fdisk_ask *ask, uint64_t result)
 {
 	assert(ask);
@@ -171,6 +264,12 @@ int fdisk_ask_number_set_result(struct fdisk_ask *ask, uint64_t result)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_get_base:
+ * @ask: ask instance
+ *
+ * Returns: base when user specify number in relative notation (+size)
+ */
 uint64_t fdisk_ask_number_get_base(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -185,7 +284,12 @@ int fdisk_ask_number_set_base(struct fdisk_ask *ask, uint64_t base)
 	return 0;
 }
 
-/* if numbers are not in bytes, then specify number of bytes per the unit */
+/**
+ * fdisk_ask_number_get_unit:
+ * @ask: ask instance
+ *
+ * Returns: number of bytes per the unit
+ */
 uint64_t fdisk_ask_number_get_unit(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -207,6 +311,17 @@ int fdisk_ask_number_is_relative(struct fdisk_ask *ask)
 	return ask->data.num.relative;
 }
 
+/**
+ * fdisk_ask_number_set_relative
+ * @ask: ask instance
+ * @relative: 0 or 1
+ *
+ * Inform libfdisk that user specified number in relative notation rather than
+ * by explicit number. This info allows to fdisk do some optimization (e.g.
+ * align end of partiton, etc.)
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_ask_number_set_relative(struct fdisk_ask *ask, int relative)
 {
 	assert(ask);
@@ -214,6 +329,16 @@ int fdisk_ask_number_set_relative(struct fdisk_ask *ask, int relative)
 	return 0;
 }
 
+/**
+ * fdisk_ask_number_inchars:
+ * @ask: ask instance
+ *
+ * For example for BSD is normal to address partition by chars rather than by
+ * number (first partition is 'a').
+ *
+ * Returns: 1 if number should be presented as chars
+ *
+ */
 int fdisk_ask_number_inchars(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -280,7 +405,16 @@ static char *mk_string_list(char *ptr, size_t *len, size_t *begin,
 	return ptr;
 }
 
-/* returns: 1=0 on success, < 0 on error, 1 if no free/used partition */
+/**
+ * fdisk_ask_partnum:
+ * @cxt: context
+ * @partnum: returns partition number
+ * @wantnew: 0|1
+ *
+ * High-level API to ask for used or unused partition number.
+ *
+ * Returns: 0 on success, < 0 on error, 1 if no free/used partition
+ */
 int fdisk_ask_partnum(struct fdisk_context *cxt, size_t *partnum, int wantnew)
 {
 	int rc = 0, inchar = 0;
@@ -375,11 +509,21 @@ dont_ask:
 			*partnum -= 1;
 	}
 	DBG(ASK, ul_debugobj(ask, "result: %ju [rc=%d]\n", fdisk_ask_number_get_result(ask), rc));
-	fdisk_free_ask(ask);
+	fdisk_unref_ask(ask);
 	return rc;
 }
 
-/* very basic wraper to ask numbers */
+/**
+ * fdisk_ask_number:
+ * @cxt: context
+ * @low: minimal possible number
+ * @dflt: default suggestion
+ * @high: maximal possible number
+ * @query: question string
+ * @result: returns result
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_ask_number(struct fdisk_context *cxt,
 		     uintmax_t low,
 		     uintmax_t dflt,
@@ -411,10 +555,16 @@ int fdisk_ask_number(struct fdisk_context *cxt,
 		*result = fdisk_ask_number_get_result(ask);
 
 	DBG(ASK, ul_debugobj(ask, "result: %ju [rc=%d]\n", *result, rc));
-	fdisk_free_ask(ask);
+	fdisk_unref_ask(ask);
 	return rc;
 }
 
+/**
+ * fdisk_ask_string_get_result:
+ * @ask: ask instance
+ *
+ * Returns: pointer to dialog result
+ */
 char *fdisk_ask_string_get_result(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -422,8 +572,15 @@ char *fdisk_ask_string_get_result(struct fdisk_ask *ask)
 	return ask->data.str.result;
 }
 
-/*
- * The @result has to be poiter to the allocated buffer.
+/**
+ * fdisk_ask_string_set_result:
+ * @ask: ask instance
+ * @result: pointer to allocated buffer with string
+ *
+ * You don't have to care about the @result deallocation, libfdisk is going to
+ * deallocate the result when destroy @ask instance.
+ *
+ * Returns: 0 on success, <0 on error
  */
 int fdisk_ask_string_set_result(struct fdisk_ask *ask, char *result)
 {
@@ -432,8 +589,15 @@ int fdisk_ask_string_set_result(struct fdisk_ask *ask, char *result)
 	return 0;
 }
 
-/*
- * Don't forget to deallocate @result.
+/**
+ * fdisk_ask_string:
+ * @cxt: context:
+ * @query: question string
+ * @result: returns allocated buffer
+ *
+ * High-level API to ask for strings. Don't forget to deallocate the @result.
+ *
+ * Returns: 0 on success, <0 on error.
  */
 int fdisk_ask_string(struct fdisk_context *cxt,
 		     const char *query,
@@ -457,10 +621,20 @@ int fdisk_ask_string(struct fdisk_context *cxt,
 		*result = fdisk_ask_string_get_result(ask);
 
 	DBG(ASK, ul_debugobj(ask, "result: %s [rc=%d]\n", *result, rc));
-	fdisk_free_ask(ask);
+	fdisk_unref_ask(ask);
 	return rc;
 }
 
+/**
+ * fdisk_ask_yesno:
+ * @cxt: context
+ * @query: question string
+ * @result: returns 0 (no) or 1 (yes)
+ *
+ * Hight-level API to ask Yes/No questions
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_ask_yesno(struct fdisk_context *cxt,
 		     const char *query,
 		     int *result)
@@ -480,21 +654,34 @@ int fdisk_ask_yesno(struct fdisk_context *cxt,
 	if (!rc)
 		rc = fdisk_do_ask(cxt, ask);
 	if (!rc)
-		*result = fdisk_ask_yesno_get_result(ask);
+		*result = fdisk_ask_yesno_get_result(ask) == 1 ? 1 : 0;
 
 	DBG(ASK, ul_debugobj(ask, "result: %d [rc=%d]\n", *result, rc));
-	fdisk_free_ask(ask);
+	fdisk_unref_ask(ask);
 	return rc;
 }
 
-uint64_t fdisk_ask_yesno_get_result(struct fdisk_ask *ask)
+/**
+ * fdisk_ask_yesno_get_result:
+ * @ask: ask instance
+ *
+ * Returns: 0 or 1
+ */
+int fdisk_ask_yesno_get_result(struct fdisk_ask *ask)
 {
 	assert(ask);
 	assert(fdisk_is_ask(ask, YESNO));
 	return ask->data.yesno.result;
 }
 
-int fdisk_ask_yesno_set_result(struct fdisk_ask *ask, uint64_t result)
+/**
+ * fdisk_ask_yesno_set_result:
+ * @ask: ask instance
+ * @result: 1 or 0
+ *
+ * Returns: 0 on success, <0 on error
+ */
+int fdisk_ask_yesno_set_result(struct fdisk_ask *ask, int result)
 {
 	assert(ask);
 	ask->data.yesno.result = result;
@@ -512,6 +699,12 @@ int fdisk_ask_menu_set_default(struct fdisk_ask *ask, int dfl)
 	return 0;
 }
 
+/**
+ * fdisk_ask_menu_get_default:
+ * @ask: ask instance
+ *
+ * Returns: default menu item key
+ */
 int fdisk_ask_menu_get_default(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -519,6 +712,13 @@ int fdisk_ask_menu_get_default(struct fdisk_ask *ask)
 	return ask->data.menu.dfl;
 }
 
+/**
+ * fdisk_ask_menu_set_result:
+ * @ask: ask instance
+ * @key: result
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_ask_menu_set_result(struct fdisk_ask *ask, int key)
 {
 	assert(ask);
@@ -529,6 +729,13 @@ int fdisk_ask_menu_set_result(struct fdisk_ask *ask, int key)
 
 }
 
+/**
+ * fdisk_ask_menu_get_result:
+ * @ask: ask instance
+ * @key: returns selected menu item key
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_ask_menu_get_result(struct fdisk_ask *ask, int *key)
 {
 	assert(ask);
@@ -538,7 +745,16 @@ int fdisk_ask_menu_get_result(struct fdisk_ask *ask, int *key)
 	return 0;
 }
 
-/* returns: 0 = success, <0 = error, >0 = idx out-of-range */
+/**
+ * fdisk_ask_menu_get_item:
+ * @ask: ask menu instance
+ * @idx: wanted menu item index
+ * @key: returns key of the menu item
+ * @name: returns name of the menu item
+ * @desc: returns description of the menu item
+ *
+ * Returns: 0 on success, <0 on error, >0 if idx out-of-range
+ */
 int fdisk_ask_menu_get_item(struct fdisk_ask *ask, size_t idx, int *key,
 			    const char **name, const char **desc)
 {
@@ -578,6 +794,12 @@ static void fdisk_ask_menu_reset_items(struct fdisk_ask *ask)
 	}
 }
 
+/**
+ * fdisk_ask_menu_get_nitems:
+ * @ask: ask instance
+ *
+ * Returns: number of menu items
+ */
 size_t fdisk_ask_menu_get_nitems(struct fdisk_ask *ask)
 {
 	struct ask_menuitem *mi;
@@ -627,6 +849,12 @@ int fdisk_ask_menu_add_item(struct fdisk_ask *ask, int key,
 
 #define is_print_ask(a) (fdisk_is_ask(a, WARN) || fdisk_is_ask(a, WARNX) || fdisk_is_ask(a, INFO))
 
+/**
+ * fdisk_ask_print_get_errno:
+ * @ask: ask instance
+ *
+ * Returns: error number for warning/error messages
+ */
 int fdisk_ask_print_get_errno(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -641,6 +869,12 @@ int fdisk_ask_print_set_errno(struct fdisk_ask *ask, int errnum)
 	return 0;
 }
 
+/**
+ * fdisk_ask_print_get_mesg:
+ * @ask: ask instance
+ *
+ * Returns: pointer to message
+ */
 const char *fdisk_ask_print_get_mesg(struct fdisk_ask *ask)
 {
 	assert(ask);
@@ -657,7 +891,7 @@ int fdisk_ask_print_set_mesg(struct fdisk_ask *ask, const char *mesg)
 }
 
 static int do_vprint(struct fdisk_context *cxt, int errnum, int type,
-		 unsigned int flags, const char *fmt, va_list va)
+		     const char *fmt, va_list va)
 {
 	struct fdisk_ask *ask;
 	int rc;
@@ -675,17 +909,26 @@ static int do_vprint(struct fdisk_context *cxt, int errnum, int type,
 	}
 
 	fdisk_ask_set_type(ask, type);
-	fdisk_ask_set_flags(ask, flags);
 	fdisk_ask_print_set_mesg(ask, mesg);
 	if (errnum >= 0)
 		fdisk_ask_print_set_errno(ask, errnum);
 	rc = fdisk_do_ask(cxt, ask);
 
-	fdisk_free_ask(ask);
+	fdisk_unref_ask(ask);
 	free(mesg);
 	return rc;
 }
 
+/**
+ * fdisk_info:
+ * @cxt: context
+ * @fmt: printf-like formatted string
+ * @...: variable parametrs
+ *
+ * High-level API to print info messages,
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_info(struct fdisk_context *cxt, const char *fmt, ...)
 {
 	int rc;
@@ -693,26 +936,21 @@ int fdisk_info(struct fdisk_context *cxt, const char *fmt, ...)
 
 	assert(cxt);
 	va_start(ap, fmt);
-	rc = do_vprint(cxt, -1, FDISK_ASKTYPE_INFO, 0, fmt, ap);
+	rc = do_vprint(cxt, -1, FDISK_ASKTYPE_INFO, fmt, ap);
 	va_end(ap);
 	return rc;
 }
 
-/* "smart" version, allows to set flags for the message */
-int fdisk_sinfo(struct fdisk_context *cxt,
-		unsigned int flags, const char *fmt, ...)
-{
-	int rc;
-	va_list ap;
-
-	assert(cxt);
-	va_start(ap, fmt);
-	rc = do_vprint(cxt, -1, FDISK_ASKTYPE_INFO, flags, fmt, ap);
-	va_end(ap);
-	return rc;
-
-}
-
+/**
+ * fdisk_info:
+ * @cxt: context
+ * @fmt: printf-like formatted string
+ * @...: variable parametrs
+ *
+ * High-level API to print warning message (errno expected)
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_warn(struct fdisk_context *cxt, const char *fmt, ...)
 {
 	int rc;
@@ -720,11 +958,21 @@ int fdisk_warn(struct fdisk_context *cxt, const char *fmt, ...)
 
 	assert(cxt);
 	va_start(ap, fmt);
-	rc = do_vprint(cxt, errno, FDISK_ASKTYPE_WARN, 0, fmt, ap);
+	rc = do_vprint(cxt, errno, FDISK_ASKTYPE_WARN, fmt, ap);
 	va_end(ap);
 	return rc;
 }
 
+/**
+ * fdisk_warnx:
+ * @cxt: context
+ * @fmt: printf-like formatted string
+ * @...: variable options
+ *
+ * High-level API to print warning message
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_warnx(struct fdisk_context *cxt, const char *fmt, ...)
 {
 	int rc;
@@ -732,21 +980,21 @@ int fdisk_warnx(struct fdisk_context *cxt, const char *fmt, ...)
 
 	assert(cxt);
 	va_start(ap, fmt);
-	rc = do_vprint(cxt, -1, FDISK_ASKTYPE_WARNX, 0, fmt, ap);
+	rc = do_vprint(cxt, -1, FDISK_ASKTYPE_WARNX, fmt, ap);
 	va_end(ap);
 	return rc;
 }
 
 int fdisk_info_new_partition(
 			struct fdisk_context *cxt,
-			int num, sector_t start, sector_t stop,
+			int num, fdisk_sector_t start, fdisk_sector_t stop,
 			struct fdisk_parttype *t)
 {
 	int rc;
 	char *str = size_to_human_string(SIZE_SUFFIX_3LETTER | SIZE_SUFFIX_SPACE,
 				     (uint64_t)(stop - start + 1) * cxt->sector_size);
 
-	rc = fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
+	rc = fdisk_info(cxt,
 			_("Created a new partition %d of type '%s' and of size %s."),
 			num, t ? t->name : _("Unknown"), str);
 	free(str);
