@@ -15,7 +15,6 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <sys/param.h>
-#include <libsmartcols.h>
 
 #include "nls.h"
 #include "blkdev.h"
@@ -24,6 +23,14 @@
 #include "pt-bsd.h"
 #include "all-io.h"
 
+
+/**
+ * SECTION: bsd
+ * @title: BSD
+ * @short_description: disk label specific functions
+ *
+ */
+
 static const char *bsd_dktypenames[] = {
 	"unknown",
 	"SMD",
@@ -86,13 +93,12 @@ static int bsd_list_disklabel(struct fdisk_context *cxt);
 static int bsd_initlabel(struct fdisk_context *cxt);
 static int bsd_readlabel(struct fdisk_context *cxt);
 static void sync_disks(struct fdisk_context *cxt);
-static int bsd_set_parttype(struct fdisk_context *cxt, size_t partnum, struct fdisk_parttype *t);
 
 static inline struct fdisk_bsd_label *self_label(struct fdisk_context *cxt)
 {
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, BSD));
+	assert(fdisk_is_label(cxt, BSD));
 
 	return (struct fdisk_bsd_label *) cxt->label;
 }
@@ -101,7 +107,7 @@ static inline struct bsd_disklabel *self_disklabel(struct fdisk_context *cxt)
 {
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, BSD));
+	assert(fdisk_is_label(cxt, BSD));
 
 	return &((struct fdisk_bsd_label *) cxt->label)->bsd;
 }
@@ -111,7 +117,7 @@ static struct fdisk_parttype *bsd_partition_parttype(
 		struct bsd_partition *p)
 {
 	struct fdisk_parttype *t
-			= fdisk_get_parttype_from_code(cxt, p->p_fstype);
+		= fdisk_label_get_parttype_from_code(cxt->label, p->p_fstype);
 	return t ? : fdisk_new_unknown_parttype(p->p_fstype, NULL);
 }
 
@@ -149,7 +155,7 @@ static int bsd_assign_dos_partition(struct fdisk_context *cxt)
 	size_t i;
 
 	for (i = 0; i < 4; i++) {
-		sector_t ss;
+		fdisk_sector_t ss;
 
 		l->dos_part = fdisk_dos_get_partition(cxt->parent, i);
 
@@ -194,8 +200,29 @@ static int bsd_probe_label(struct fdisk_context *cxt)
 	return 0;		/* not found */
 }
 
+static int set_parttype(
+		struct fdisk_context *cxt,
+		size_t partnum,
+		struct fdisk_parttype *t)
+{
+	struct bsd_partition *p;
+	struct bsd_disklabel *d = self_disklabel(cxt);
+
+	if (partnum >= d->d_npartitions || !t || t->code > UINT8_MAX)
+		return -EINVAL;
+
+	p = &d->d_partitions[partnum];
+	if (t->code == p->p_fstype)
+		return 0;
+
+	p->p_fstype = t->code;
+	fdisk_label_set_changed(cxt->label, 1);
+	return 0;
+}
+
 static int bsd_add_partition(struct fdisk_context *cxt,
-			     struct fdisk_partition *pa)
+			     struct fdisk_partition *pa,
+			     size_t *partno)
 {
 	struct fdisk_bsd_label *l = self_label(cxt);
 	struct bsd_disklabel *d = self_disklabel(cxt);
@@ -219,7 +246,7 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 	 */
 	if (pa && pa->start_follow_default)
 		;
-	else if (pa && pa->start) {
+	else if (pa && fdisk_partition_has_start(pa)) {
 		if (pa->start < begin || pa->start > end)
 			return -ERANGE;
 		begin = pa->start;
@@ -229,7 +256,7 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 		if (!ask)
 			return -ENOMEM;
 		fdisk_ask_set_query(ask,
-			fdisk_context_use_cylinders(cxt) ?
+			fdisk_use_cylinders(cxt) ?
 			_("First cylinder") : _("First sector"));
 		fdisk_ask_set_type(ask, FDISK_ASKTYPE_NUMBER);
 		fdisk_ask_number_set_low(ask, fdisk_cround(cxt, begin));
@@ -238,10 +265,10 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 
 		rc = fdisk_do_ask(cxt, ask);
 		begin = fdisk_ask_number_get_result(ask);
-		fdisk_free_ask(ask);
+		fdisk_unref_ask(ask);
 		if (rc)
 			return rc;
-		if (fdisk_context_use_cylinders(cxt))
+		if (fdisk_use_cylinders(cxt))
 			begin = (begin - 1) * d->d_secpercyl;
 	}
 
@@ -250,10 +277,10 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 	 */
 	if (pa && pa->end_follow_default)
 		;
-	else if (pa && pa->size) {
+	else if (pa && fdisk_partition_has_size(pa)) {
 		if (begin + pa->size > end)
 			return -ERANGE;
-		end = begin + pa->size;
+		end = begin + pa->size - 1ULL;
 	} else {
 		/* ask user by dialog */
 		struct fdisk_ask *ask = fdisk_new_ask();
@@ -262,11 +289,11 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 			return -ENOMEM;
 		fdisk_ask_set_type(ask, FDISK_ASKTYPE_OFFSET);
 
-		if (fdisk_context_use_cylinders(cxt)) {
+		if (fdisk_use_cylinders(cxt)) {
 			fdisk_ask_set_query(ask, _("Last cylinder, +cylinders or +size{K,M,G,T,P}"));
 			fdisk_ask_number_set_unit(ask,
 				     cxt->sector_size *
-				     fdisk_context_get_units_per_sector(cxt));
+				     fdisk_get_units_per_sector(cxt));
 		} else {
 			fdisk_ask_set_query(ask, _("Last sector, +sectors or +size{K,M,G,T,P}"));
 			fdisk_ask_number_set_unit(ask,cxt->sector_size);
@@ -279,10 +306,10 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 
 		rc = fdisk_do_ask(cxt, ask);
 		end = fdisk_ask_number_get_result(ask);
-		fdisk_free_ask(ask);
+		fdisk_unref_ask(ask);
 		if (rc)
 			return rc;
-		if (fdisk_context_use_cylinders(cxt))
+		if (fdisk_use_cylinders(cxt))
 			end = end * d->d_secpercyl - 1;
 	}
 
@@ -295,12 +322,57 @@ static int bsd_add_partition(struct fdisk_context *cxt,
 	cxt->label->nparts_cur = d->d_npartitions;
 
 	if (pa && pa->type)
-		bsd_set_parttype(cxt, i, pa->type);
+		set_parttype(cxt, i, pa->type);
+
+	fdisk_label_set_changed(cxt->label, 1);
+	if (partno)
+		*partno = i;
+	return 0;
+}
+
+static int bsd_set_partition(struct fdisk_context *cxt, size_t n,
+			     struct fdisk_partition *pa)
+{
+	struct bsd_partition *p;
+	struct fdisk_bsd_label *l = self_label(cxt);
+	struct bsd_disklabel *d = self_disklabel(cxt);
+
+	if (n >= d->d_npartitions)
+		return -EINVAL;
+
+	p = &d->d_partitions[n];
+
+	/* we have to stay within parental DOS partition */
+	if (l->dos_part && (fdisk_partition_has_start(pa) ||
+			    fdisk_partition_has_size(pa))) {
+
+		fdisk_sector_t dosbegin = dos_partition_get_start(l->dos_part);
+		fdisk_sector_t dosend = dosbegin + dos_partition_get_size(l->dos_part) - 1;
+		fdisk_sector_t begin = fdisk_partition_has_start(pa) ? pa->start : p->p_offset;
+		fdisk_sector_t end = begin + (fdisk_partition_has_size(pa) ? pa->size : p->p_size) - 1;
+
+		if (begin < dosbegin || begin > dosend)
+			return -ERANGE;
+		if (end < dosbegin || end > dosend)
+			return -ERANGE;
+	}
+
+	if (pa->type) {
+		int rc = set_parttype(cxt, n, pa->type);
+		if (rc)
+			return rc;
+	}
+
+	if (fdisk_partition_has_start(pa))
+		d->d_partitions[n].p_offset = pa->start;
+	if (fdisk_partition_has_size(pa))
+		d->d_partitions[n].p_size = pa->size;
 
 	fdisk_label_set_changed(cxt->label, 1);
 	return 0;
 }
 
+
 /* Returns 0 on success, < 0 on error. */
 static int bsd_create_disklabel(struct fdisk_context *cxt)
 {
@@ -326,14 +398,14 @@ static int bsd_create_disklabel(struct fdisk_context *cxt)
 
 	rc = bsd_initlabel(cxt);
 	if (!rc) {
-		int org = fdisk_context_display_details(cxt);
+		int org = fdisk_is_details(cxt);
 
 		cxt->label->nparts_cur = d->d_npartitions;
 		cxt->label->nparts_max = BSD_MAXPARTITIONS;
 
-		fdisk_context_enable_details(cxt, 1);
+		fdisk_enable_details(cxt, 1);
 		bsd_list_disklabel(cxt);
-		fdisk_context_enable_details(cxt, org);
+		fdisk_enable_details(cxt, org);
 	}
 
 	return rc;
@@ -364,9 +436,9 @@ static int bsd_list_disklabel(struct fdisk_context *cxt)
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, BSD));
+	assert(fdisk_is_label(cxt, BSD));
 
-	if (fdisk_context_display_details(cxt)) {
+	if (fdisk_is_details(cxt)) {
 		fdisk_info(cxt, "# %s:", cxt->dev_path);
 
 		if ((unsigned) d->d_type < BSD_DKMAXTYPES)
@@ -410,7 +482,7 @@ static int bsd_get_partition(struct fdisk_context *cxt, size_t n,
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, BSD));
+	assert(fdisk_is_label(cxt, BSD));
 
 	if (n >= d->d_npartitions)
 		return -EINVAL;
@@ -421,13 +493,12 @@ static int bsd_get_partition(struct fdisk_context *cxt, size_t n,
 	if (!pa->used)
 		return 0;
 
-	if (fdisk_context_use_cylinders(cxt) && d->d_secpercyl) {
+	if (fdisk_use_cylinders(cxt) && d->d_secpercyl) {
 		pa->start_post = p->p_offset % d->d_secpercyl ? '*' : ' ';
 		pa->end_post = (p->p_offset + p->p_size) % d->d_secpercyl ? '*' : ' ';
 	}
 
 	pa->start = p->p_offset;
-	pa->end = p->p_offset + p->p_size - 1;
 	pa->size = p->p_size;
 	pa->type = bsd_partition_parttype(cxt, p);
 
@@ -463,6 +534,14 @@ static uint16_t ask_uint16(struct fdisk_context *cxt,
 	return dflt;
 }
 
+/**
+ * fdisk_bsd_edit_disklabel:
+ * @cxt: context
+ *
+ * Edits fields in BSD disk label.
+ *
+ * Returns: 0 on success, <0 on error
+ */
 int fdisk_bsd_edit_disklabel(struct fdisk_context *cxt)
 {
 	struct bsd_disklabel *d = self_disklabel(cxt);
@@ -510,12 +589,17 @@ static int bsd_get_bootstrap(struct fdisk_context *cxt,
 		return -errno;
 	}
 
-	fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
-			_("The bootstrap file %s successfully loaded."), path);
+	fdisk_info(cxt, _("The bootstrap file %s successfully loaded."), path);
 	close (fd);
 	return 0;
 }
 
+/**
+ * fdisk_bsd_write_bootstrap:
+ * @cxt: context
+ *
+ * Install bootstrap file to the BSD device
+ */
 int fdisk_bsd_write_bootstrap(struct fdisk_context *cxt)
 {
 	struct bsd_disklabel dl, *d = self_disklabel(cxt);
@@ -524,7 +608,7 @@ int fdisk_bsd_write_bootstrap(struct fdisk_context *cxt)
 	char buf[BUFSIZ];
 	char *res, *dp, *p;
 	int rc;
-	sector_t sector;
+	fdisk_sector_t sector;
 
 	snprintf(buf, sizeof(buf),
 		_("Bootstrap: %1$sboot -> boot%1$s (default %1$s)"),
@@ -582,8 +666,7 @@ int fdisk_bsd_write_bootstrap(struct fdisk_context *cxt)
 		goto done;
 	}
 
-	fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
-			_("Bootstrap installed on %s."), cxt->dev_path);
+	fdisk_info(cxt, _("Bootstrap installed on %s."), cxt->dev_path);
 	sync_disks(cxt);
 
 	rc = 0;
@@ -754,8 +837,7 @@ static int bsd_write_disklabel(struct fdisk_context *cxt)
 	}
 	sync_disks(cxt);
 
-	fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
-			_("Disklabel written to %s."), cxt->dev_path);
+	fdisk_info(cxt, _("Disklabel written to %s."), cxt->dev_path);
 	return 0;
 }
 
@@ -786,8 +868,13 @@ static int bsd_translate_fstype (int linux_type)
 	return BSD_FS_OTHER;
 }
 
-/*
- * link partition from parent (DOS) to nested BSD partition table
+/**
+ * fdisk_bsd_link_partition:
+ * @cxt: context
+ *
+ * Links partition from parent (DOS) to nested BSD partition table.
+ *
+ * Returns: 0 on success, <0 on error
  */
 int fdisk_bsd_link_partition(struct fdisk_context *cxt)
 {
@@ -796,7 +883,7 @@ int fdisk_bsd_link_partition(struct fdisk_context *cxt)
 	struct dos_partition *p;
 	struct bsd_disklabel *d = self_disklabel(cxt);
 
-	if (!cxt->parent || !fdisk_is_disklabel(cxt->parent, DOS)) {
+	if (!cxt->parent || !fdisk_is_label(cxt->parent, DOS)) {
 		fdisk_warnx(cxt, _("BSD label is not nested within a DOS partition."));
 		return -EINVAL;
 	}
@@ -825,31 +912,11 @@ int fdisk_bsd_link_partition(struct fdisk_context *cxt)
 	cxt->label->nparts_cur = d->d_npartitions;
 	fdisk_label_set_changed(cxt->label, 1);
 
-	fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
-			_("BSD partition '%c' linked to DOS partition %zu."),
+	fdisk_info(cxt, _("BSD partition '%c' linked to DOS partition %zu."),
 			'a' + (int) i, k + 1);
 	return 0;
 }
 
-static int bsd_set_parttype(
-		struct fdisk_context *cxt,
-		size_t partnum,
-		struct fdisk_parttype *t)
-{
-	struct bsd_partition *p;
-	struct bsd_disklabel *d = self_disklabel(cxt);
-
-	if (partnum >= d->d_npartitions || !t || t->type > UINT8_MAX)
-		return -EINVAL;
-
-	p = &d->d_partitions[partnum];
-	if (t->type == p->p_fstype)
-		return 0;
-
-	p->p_fstype = t->type;
-	fdisk_label_set_changed(cxt->label, 1);
-	return 0;
-}
 
 static int bsd_partition_is_used(
 		struct fdisk_context *cxt,
@@ -870,27 +937,27 @@ static const struct fdisk_label_operations bsd_operations =
 	.list		= bsd_list_disklabel,
 	.write		= bsd_write_disklabel,
 	.create		= bsd_create_disklabel,
-	.part_delete	= bsd_delete_part,
 
+	.del_part	= bsd_delete_part,
 	.get_part	= bsd_get_partition,
+	.set_part	= bsd_set_partition,
 	.add_part	= bsd_add_partition,
 
-	.part_set_type	= bsd_set_parttype,
 	.part_is_used   = bsd_partition_is_used,
 };
 
-static const struct fdisk_column bsd_columns[] =
+static const struct fdisk_field bsd_fields[] =
 {
-	{ FDISK_COL_DEVICE,	N_("Slice"),	  1,	0 },
-	{ FDISK_COL_START,	N_("Start"),	  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_END,	N_("End"),	  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_SECTORS,	N_("Sectors"),    5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_CYLINDERS,	N_("Cylinders"),  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_SIZE,	N_("Size"),	  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_TYPE,	N_("Type"),	  8,	0 },
-	{ FDISK_COL_FSIZE,	N_("Fsize"),	  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_BSIZE,	N_("Bsize"),	  5,	SCOLS_FL_RIGHT },
-	{ FDISK_COL_CPG,	N_("Cpg"),	  5,	SCOLS_FL_RIGHT }
+	{ FDISK_FIELD_DEVICE,	N_("Slice"),	  1,	0 },
+	{ FDISK_FIELD_START,	N_("Start"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_END,	N_("End"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_SECTORS,	N_("Sectors"),    5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_CYLINDERS,N_("Cylinders"),  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_SIZE,	N_("Size"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_TYPE,	N_("Type"),	  8,	0 },
+	{ FDISK_FIELD_FSIZE,	N_("Fsize"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_BSIZE,	N_("Bsize"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_CPG,	N_("Cpg"),	  5,	FDISK_FIELDFL_NUMBER }
 };
 
 /*
@@ -913,10 +980,10 @@ struct fdisk_label *fdisk_new_bsd_label(struct fdisk_context *cxt)
 	lb->id = FDISK_DISKLABEL_BSD;
 	lb->op = &bsd_operations;
 	lb->parttypes = bsd_fstypes;
-	lb->nparttypes = ARRAY_SIZE(bsd_fstypes);
+	lb->nparttypes = ARRAY_SIZE(bsd_fstypes) - 1;
 
-	lb->columns = bsd_columns;
-	lb->ncolumns = ARRAY_SIZE(bsd_columns);
+	lb->fields = bsd_fields;
+	lb->nfields = ARRAY_SIZE(bsd_fields);
 
 	lb->flags |= FDISK_LABEL_FL_INCHARS_PARTNO;
 	lb->flags |= FDISK_LABEL_FL_REQUIRE_GEOMETRY;
