@@ -37,20 +37,21 @@
 /*
  * Debug
  */
-#define FDISK_DEBUG_INIT	(1 << 1)
-#define FDISK_DEBUG_CXT		(1 << 2)
-#define FDISK_DEBUG_LABEL       (1 << 3)
-#define FDISK_DEBUG_ASK         (1 << 4)
-#define FDISK_DEBUG_FRONTEND	(1 << 5)
-#define FDISK_DEBUG_PART	(1 << 6)
-#define FDISK_DEBUG_PARTTYPE	(1 << 7)
-#define FDISK_DEBUG_TAB		(1 << 8)
-#define FDISK_DEBUG_ALL		0xFFFF
+#define LIBFDISK_DEBUG_HELP	(1 << 0)
+#define LIBFDISK_DEBUG_INIT	(1 << 1)
+#define LIBFDISK_DEBUG_CXT	(1 << 2)
+#define LIBFDISK_DEBUG_LABEL    (1 << 3)
+#define LIBFDISK_DEBUG_ASK      (1 << 4)
+#define LIBFDISK_DEBUG_PART	(1 << 6)
+#define LIBFDISK_DEBUG_PARTTYPE	(1 << 7)
+#define LIBFDISK_DEBUG_TAB	(1 << 8)
+#define LIBFDISK_DEBUG_SCRIPT	(1 << 9)
+#define LIBFDISK_DEBUG_ALL	0xFFFF
 
 UL_DEBUG_DECLARE_MASK(libfdisk);
-#define DBG(m, x)	__UL_DBG(libfdisk, FDISK_DEBUG_, m, x)
-#define ON_DBG(m, x)	__UL_DBG_CALL(libfdisk, FDISK_DEBUG_, m, x)
-#define DBG_FLUSH	__UL_DBG_FLUSH(libfdisk, FDISK_DEBUG_)
+#define DBG(m, x)	__UL_DBG(libfdisk, LIBFDISK_DEBUG_, m, x)
+#define ON_DBG(m, x)	__UL_DBG_CALL(libfdisk, LIBFDISK_DEBUG_, m, x)
+#define DBG_FLUSH	__UL_DBG_FLUSH(libfdisk, LIBFDISK_DEBUG_)
 
 #ifdef TEST_PROGRAM
 struct fdisk_test {
@@ -64,9 +65,6 @@ extern int fdisk_run_test(struct fdisk_test *tests, int argc, char *argv[]);
 #endif
 
 
-typedef unsigned long long sector_t;
-
-
 /*
  * Generic iterator
  */
@@ -97,11 +95,12 @@ struct fdisk_iter {
  * Partition types
  */
 struct fdisk_parttype {
-	unsigned int	type;		/* type as number or zero */
-	const char	*name;		/* description */
+	unsigned int	code;		/* type as number or zero */
+	char		*name;		/* description */
 	char		*typestr;	/* type as string or NULL */
 
 	unsigned int	flags;		/* FDISK_PARTTYPE_* flags */
+	int		refcount;	/* reference counter for allocated types */
 };
 
 enum {
@@ -110,18 +109,20 @@ enum {
 	FDISK_PARTTYPE_ALLOCATED	= (1 << 3)
 };
 
-#define fdisk_parttype_is_unknown(_x)	((_x) && ((_x)->flags & FDISK_PARTTYPE_UNKNONW))
 #define fdisk_parttype_is_invisible(_x)	((_x) && ((_x)->flags & FDISK_PARTTYPE_INVISIBLE))
 #define fdisk_parttype_is_allocated(_x)	((_x) && ((_x)->flags & FDISK_PARTTYPE_ALLOCATED))
 
 struct fdisk_partition {
 	int		refcount;		/* reference counter */
+
 	size_t		partno;			/* partition number */
 	size_t		parent_partno;		/* for logical partitions */
 
-	uint64_t	start;			/* first sectors */
-	uint64_t	end;			/* last sector */
-	uint64_t	size;			/* size in sectors */
+	fdisk_sector_t	start;			/* first sectors */
+	fdisk_sector_t	size;			/* size in sectors */
+
+	int		movestart;		/* FDISK_MOVE_* (scripts only) */
+	int		resize;			/* FDISK_RESIZE_* (scripts only) */
 
 	char		*name;			/* partition name */
 	char		*uuid;			/* partition UUID */
@@ -139,21 +140,35 @@ struct fdisk_partition {
 	uint64_t	bsize;
 	uint64_t	cpg;
 
-	char		boot;			/* is bootable (MBS only) */
-	char		*start_addr;		/* start C/H/S in string */
-	char		*end_addr;		/* end C/H/S in string */
+	char		*start_chs;		/* start C/H/S in string */
+	char		*end_chs;		/* end C/H/S in string */
 
-	unsigned int	partno_follow_default : 1,	/* use default partno */
-			start_follow_default : 1,	/* use default start */
+	unsigned int	boot;			/* MBR: bootable */
+
+	unsigned int	container : 1,			/* container partition (e.g. extended partition) */
 			end_follow_default : 1,		/* use default end */
-			freespace : 1,		/* this is free space */
-			container : 1,		/* container partition (e.g. extended partition) */
-			wholedisk : 1,		/* special system partition */
-			used   : 1;		/* partition already used */
+			freespace : 1,			/* this is free space */
+			partno_follow_default : 1,	/* use default partno */
+			size_explicit : 1,		/* don't align the size */
+			start_follow_default : 1,	/* use default start */
+			used : 1,			/* partition already used */
+			wholedisk : 1;			/* special system partition */
 };
 
-#define FDISK_EMPTY_PARTNO	((size_t) -1)
-#define FDISK_EMPTY_PARTITION	{ .partno = FDISK_EMPTY_PARTNO }
+enum {
+	FDISK_MOVE_NONE = 0,
+	FDISK_MOVE_DOWN = -1,
+	FDISK_MOVE_UP = 1
+};
+
+enum {
+	FDISK_RESIZE_NONE = 0,
+	FDISK_RESIZE_REDUCE = -1,
+	FDISK_RESIZE_ENLARGE = 1
+};
+
+#define FDISK_INIT_UNDEF(_x)	((_x) = (__typeof__(_x)) -1)
+#define FDISK_IS_UNDEF(_x)	((_x) == (__typeof__(_x)) -1)
 
 struct fdisk_table {
 	struct list_head	parts;		/* partitions */
@@ -166,8 +181,8 @@ struct fdisk_table {
  */
 struct fdisk_geometry {
 	unsigned int heads;
-	sector_t sectors;
-	sector_t cylinders;
+	fdisk_sector_t sectors;
+	fdisk_sector_t cylinders;
 };
 
 /*
@@ -185,7 +200,8 @@ struct fdisk_label_operations {
 	/* list disklabel details */
 	int (*list)(struct fdisk_context *cxt);
 	/* returns offset and size of the 'n' part of the PT */
-	int (*locate)(struct fdisk_context *cxt, int n, const char **name, off_t *offset, size_t *size);
+	int (*locate)(struct fdisk_context *cxt, int n, const char **name,
+		      uint64_t *offset, size_t *size);
 	/* reorder partitions */
 	int (*reorder)(struct fdisk_context *cxt);
 
@@ -194,28 +210,23 @@ struct fdisk_label_operations {
 	/* set disk label ID */
 	int (*set_id)(struct fdisk_context *cxt);
 
-	/* new partition */
-	int (*add_part)(struct fdisk_context *cxt, struct fdisk_partition *pa);
 
+	/* new partition */
+	int (*add_part)(struct fdisk_context *cxt, struct fdisk_partition *pa,
+						size_t *partno);
 	/* delete partition */
-	int (*part_delete)(struct fdisk_context *cxt,
-						size_t partnum);
-	/* get partition type */
-	struct fdisk_parttype *(*part_get_type)(struct fdisk_context *cxt,
-						size_t partnum);
-	/* set partition type */
-	int (*part_set_type)(struct fdisk_context *cxt,
-						size_t partnum,
-						struct fdisk_parttype *t);
+	int (*del_part)(struct fdisk_context *cxt, size_t partnum);
+
+	/* fill in partition struct */
+	int (*get_part)(struct fdisk_context *cxt, size_t n,
+						struct fdisk_partition *pa);
+	/* modify partition */
+	int (*set_part)(struct fdisk_context *cxt, size_t n,
+						struct fdisk_partition *pa);
 
 	/* return state of the partition */
 	int (*part_is_used)(struct fdisk_context *cxt, size_t partnum);
 
-	/* fill in partition struct */
-	int (*get_part)(struct fdisk_context *cxt,
-						size_t n,
-						struct fdisk_partition *pa);
-
 	int (*part_toggle_flag)(struct fdisk_context *cxt, size_t i, unsigned long flag);
 
 	/* refresh alignment setting */
@@ -229,21 +240,20 @@ struct fdisk_label_operations {
 };
 
 /*
- * fdisk_label_operations->list() output column
+ * The fields describes how to display libfdisk_partition
  */
-struct fdisk_column {
-	int		id;		/* FDISK_COL_* */
-	const char	*name;		/* column header */
-	double		width;
-	int		scols_flags;	/* SCOLS_FL_* */
-
-	int		flags;		/* FDISK_COLFL_* */
+struct fdisk_field {
+	int		id;		/* FDISK_FIELD_* */
+	const char	*name;		/* field name */
+	double		width;		/* field width (compatible with libsmartcols whint) */
+	int		flags;		/* FDISK_FIELDFL_* */
 };
 
 /* note that the defauls is to display a column always */
 enum {
-	FDISK_COLFL_DETAIL	= (1 << 1),	/* only display if fdisk_context_display_details() */
-	FDISK_COLFL_EYECANDY	= (1 << 2),	/* don't display if fdisk_context_display_details() */
+	FDISK_FIELDFL_DETAIL	= (1 << 1),	/* only display if fdisk_is_details() */
+	FDISK_FIELDFL_EYECANDY	= (1 << 2),	/* don't display if fdisk_is_details() */
+	FDISK_FIELDFL_NUMBER	= (1 << 3),	/* column display numbers */
 };
 
 /*
@@ -263,8 +273,8 @@ struct fdisk_label {
 	unsigned int		changed:1,	/* label has been modified */
 				disabled:1;	/* this driver is disabled at all */
 
-	const struct fdisk_column *columns;	/* all possible columns */
-	size_t			ncolumns;
+	const struct fdisk_field *fields;	/* all possible fields */
+	size_t			nfields;
 
 	const struct fdisk_label_operations *op;
 };
@@ -298,7 +308,8 @@ struct ask_menuitem {
 struct fdisk_ask {
 	int		type;		/* FDISK_ASKTYPE_* */
 	char		*query;
-	unsigned int	flags;
+
+	int		refcount;
 
 	union {
 		/* FDISK_ASKTYPE_{NUMBER,OFFSET} */
@@ -338,6 +349,7 @@ struct fdisk_ask {
 struct fdisk_context {
 	int dev_fd;         /* device descriptor */
 	char *dev_path;     /* device path */
+	int refcount;
 
 	unsigned char *firstsector; /* buffer with master boot record */
 	unsigned long firstsector_bufsz;
@@ -353,15 +365,18 @@ struct fdisk_context {
 	unsigned int readonly : 1,		/* don't write to the device */
 		     display_in_cyl_units : 1,	/* for obscure labels */
 		     display_details : 1,	/* expert display mode */
+		     protect_bootbits : 1,	/* don't zeroize fll irst sector */
 		     listonly : 1;		/* list partition, nothing else */
 
+	int sizeunit;				/* SIZE fields, FDISK_SIZEUNIT_* */
+
 	/* alignment */
 	unsigned long grain;		/* alignment unit */
-	sector_t first_lba;		/* recommended begin of the first partition */
-	sector_t last_lba;		/* recomennded end of last partition */
+	fdisk_sector_t first_lba;		/* recommended begin of the first partition */
+	fdisk_sector_t last_lba;		/* recomennded end of last partition */
 
 	/* geometry */
-	sector_t total_sectors;	/* in logical sectors */
+	fdisk_sector_t total_sectors;	/* in logical sectors */
 	struct fdisk_geometry geom;
 
 	/* user setting to overwrite device default */
@@ -379,39 +394,22 @@ struct fdisk_context {
 	void	*ask_data;		/* ask_cb() data */
 
 	struct fdisk_context	*parent;	/* for nested PT */
+	struct fdisk_script	*script;	/* what we want to follow */
 };
 
+/* partition.c */
+int fdisk_partition_next_partno(struct fdisk_partition *pa,
+				       struct fdisk_context *cxt,
+				       size_t *n);
+
 /* context.c */
-extern int __fdisk_context_switch_label(struct fdisk_context *cxt,
+extern int __fdisk_switch_label(struct fdisk_context *cxt,
 				    struct fdisk_label *lb);
-
-extern int fdisk_context_enable_listonly(struct fdisk_context *cxt, int enable);
-extern int fdisk_context_listonly(struct fdisk_context *cxt);
-
+extern int fdisk_missing_geometry(struct fdisk_context *cxt);
 
 /* alignment.c */
-extern sector_t fdisk_scround(struct fdisk_context *cxt, sector_t num);
-extern sector_t fdisk_cround(struct fdisk_context *cxt, sector_t num);
-
-extern sector_t fdisk_topology_get_first_lba(struct fdisk_context *cxt);
-extern unsigned long fdisk_topology_get_grain(struct fdisk_context *cxt);
-
-extern void fdisk_warn_alignment(struct fdisk_context *cxt,
-				 sector_t lba, int partition);
-
-
-#define FDISK_ALIGN_UP		1
-#define FDISK_ALIGN_DOWN	2
-#define FDISK_ALIGN_NEAREST	3
-
-extern sector_t fdisk_align_lba(struct fdisk_context *cxt, sector_t lba, int direction);
-extern sector_t fdisk_align_lba_in_range(struct fdisk_context *cxt, sector_t lba,
-					 sector_t start, sector_t stop);
-
-
-extern int fdisk_override_geometry(struct fdisk_context *cxt,
-		            unsigned int cylinders, unsigned int heads,
-                            unsigned int sectors);
+fdisk_sector_t fdisk_scround(struct fdisk_context *cxt, fdisk_sector_t num);
+fdisk_sector_t fdisk_cround(struct fdisk_context *cxt, fdisk_sector_t num);
 
 extern int fdisk_discover_geometry(struct fdisk_context *cxt);
 extern int fdisk_discover_topology(struct fdisk_context *cxt);
@@ -420,22 +418,41 @@ extern int fdisk_apply_user_device_properties(struct fdisk_context *cxt);
 extern void fdisk_zeroize_device_properties(struct fdisk_context *cxt);
 
 /* utils.c */
-extern int fdisk_init_firstsector_buffer(struct fdisk_context *cxt);
+extern int fdisk_init_firstsector_buffer(struct fdisk_context *cxt,
+			unsigned int protect_off, unsigned int protect_size);
 extern int fdisk_read_firstsector(struct fdisk_context *cxt);
 extern char *fdisk_partname(const char *dev, size_t partno);
 
 /* label.c */
 extern int fdisk_probe_labels(struct fdisk_context *cxt);
 extern void fdisk_deinit_label(struct fdisk_label *lb);
-extern const struct fdisk_column *fdisk_label_get_column(
-					struct fdisk_label *lb, int id);
 
 /* ask.c */
-extern int fdisk_ask_partnum(struct fdisk_context *cxt, size_t *partnum, int wantnew);
-
-extern int fdisk_info_new_partition(
+struct fdisk_ask *fdisk_new_ask(void);
+void fdisk_reset_ask(struct fdisk_ask *ask);
+int fdisk_ask_set_query(struct fdisk_ask *ask, const char *str);
+int fdisk_ask_set_type(struct fdisk_ask *ask, int type);
+int fdisk_do_ask(struct fdisk_context *cxt, struct fdisk_ask *ask);
+int fdisk_ask_number_set_range(struct fdisk_ask *ask, const char *range);
+int fdisk_ask_number_set_default(struct fdisk_ask *ask, uint64_t dflt);
+int fdisk_ask_number_set_low(struct fdisk_ask *ask, uint64_t low);
+int fdisk_ask_number_set_high(struct fdisk_ask *ask, uint64_t high);
+int fdisk_ask_number_set_base(struct fdisk_ask *ask, uint64_t base);
+int fdisk_ask_number_set_unit(struct fdisk_ask *ask, uint64_t unit);
+int fdisk_ask_number_is_relative(struct fdisk_ask *ask);
+int fdisk_ask_menu_set_default(struct fdisk_ask *ask, int dfl);
+int fdisk_ask_menu_add_item(struct fdisk_ask *ask, int key,
+			const char *name, const char *desc);
+int fdisk_ask_print_set_errno(struct fdisk_ask *ask, int errnum);
+int fdisk_ask_print_set_mesg(struct fdisk_ask *ask, const char *mesg);
+int fdisk_info_new_partition(
 			struct fdisk_context *cxt,
-			int num, sector_t start, sector_t stop,
+			int num, fdisk_sector_t start, fdisk_sector_t stop,
 			struct fdisk_parttype *t);
 
+/* dos.c */
+extern struct dos_partition *fdisk_dos_get_partition(
+				struct fdisk_context *cxt,
+				size_t i);
+
 #endif /* _LIBFDISK_PRIVATE_H */
