@@ -20,6 +20,14 @@
 #include "pt-sun.h"
 #include "all-io.h"
 
+
+/**
+ * SECTION: sun
+ * @title: SUN
+ * @short_description: disk label specific functions
+ *
+ */
+
 /*
  * in-memory fdisk SUN stuff
  */
@@ -53,7 +61,7 @@ static inline struct sun_disklabel *self_disklabel(struct fdisk_context *cxt)
 {
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	return ((struct fdisk_sun_label *) cxt->label)->header;
 }
@@ -63,16 +71,17 @@ static inline struct fdisk_sun_label *self_label(struct fdisk_context *cxt)
 {
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	return (struct fdisk_sun_label *) cxt->label;
 }
 
-static void set_sun_partition(struct fdisk_context *cxt, size_t i,
+static void set_partition(struct fdisk_context *cxt, size_t i,
 		uint32_t start,uint32_t stop, uint16_t sysid)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
-	struct fdisk_parttype *t = fdisk_get_parttype_from_code(cxt, sysid);
+	struct fdisk_parttype *t =
+			fdisk_label_get_parttype_from_code(cxt->label, sysid);
 
 	sunlabel->vtoc.infos[i].id = cpu_to_be16(sysid);
 	sunlabel->vtoc.infos[i].flags = cpu_to_be16(0);
@@ -108,7 +117,7 @@ static int sun_probe_label(struct fdisk_context *cxt)
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	/* map first sector to header */
 	sun = (struct fdisk_sun_label *) cxt->label;
@@ -193,13 +202,17 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 	unsigned int ndiv;
 	struct fdisk_sun_label *sun;		/* libfdisk sun handler */
 	struct sun_disklabel *sunlabel;	/* on disk data */
+	int rc = 0;
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	/* map first sector to header */
-	fdisk_zeroize_firstsector(cxt);
+	rc = fdisk_init_firstsector_buffer(cxt, 0, 0);
+	if (rc)
+		return rc;
+
 	sun = (struct fdisk_sun_label *) cxt->label;
 	sun->header = (struct sun_disklabel *) cxt->firstsector;
 
@@ -214,11 +227,11 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 
 #ifdef HDIO_GETGEO
 	if (cxt->geom.heads && cxt->geom.sectors) {
-		sector_t llsectors;
+		fdisk_sector_t llsectors;
 
-		if (blkdev_get_sectors(cxt->dev_fd, &llsectors) == 0) {
+		if (blkdev_get_sectors(cxt->dev_fd, (unsigned long long *) &llsectors) == 0) {
 			int sec_fac = cxt->sector_size / 512;
-			sector_t llcyls;
+			fdisk_sector_t llcyls;
 
 			llcyls = llsectors / (cxt->geom.heads * cxt->geom.sectors * sec_fac);
 			cxt->geom.cylinders = llcyls;
@@ -247,25 +260,30 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 	sunlabel->ncyl   = cpu_to_be16(cxt->geom.cylinders);
 
 	snprintf((char *) sunlabel->label_id, sizeof(sunlabel->label_id),
-		 "Linux cyl %llu alt %u hd %u sec %llu",
-		 cxt->geom.cylinders, be16_to_cpu(sunlabel->acyl),
-		 cxt->geom.heads, cxt->geom.sectors);
+		 "Linux cyl %ju alt %u hd %u sec %ju",
+		 (uintmax_t) cxt->geom.cylinders,
+		 be16_to_cpu(sunlabel->acyl),
+		 cxt->geom.heads,
+		 (uintmax_t) cxt->geom.sectors);
 
 	if (cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors >= 150 * 2048) {
 	        ndiv = cxt->geom.cylinders - (50 * 2048 / (cxt->geom.heads * cxt->geom.sectors)); /* 50M swap */
 	} else
 	        ndiv = cxt->geom.cylinders * 2 / 3;
 
-	set_sun_partition(cxt, 0, 0, ndiv * cxt->geom.heads * cxt->geom.sectors,
+	/* create the default layout only if no-script defined */
+	if (!cxt->script) {
+		set_partition(cxt, 0, 0, ndiv * cxt->geom.heads * cxt->geom.sectors,
 			  SUN_TAG_LINUX_NATIVE);
-	set_sun_partition(cxt, 1, ndiv * cxt->geom.heads * cxt->geom.sectors,
+		set_partition(cxt, 1, ndiv * cxt->geom.heads * cxt->geom.sectors,
 			  cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors,
 			  SUN_TAG_LINUX_SWAP);
-	sunlabel->vtoc.infos[1].flags |= cpu_to_be16(SUN_FLAG_UNMNT);
+		sunlabel->vtoc.infos[1].flags |= cpu_to_be16(SUN_FLAG_UNMNT);
 
-	set_sun_partition(cxt, 2, 0,
+		set_partition(cxt, 2, 0,
 			  cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors,
 			  SUN_TAG_WHOLEDISK);
+	}
 
 	{
 		unsigned short *ush = (unsigned short *)sunlabel;
@@ -278,8 +296,7 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 	fdisk_label_set_changed(cxt->label, 1);
 	cxt->label->nparts_cur = count_used_partitions(cxt);
 
-	fdisk_sinfo(cxt, FDISK_INFO_SUCCESS,
-			_("Created a new Sun disklabel."));
+	fdisk_info(cxt, _("Created a new Sun disklabel."));
 	return 0;
 }
 
@@ -290,7 +307,7 @@ static int sun_toggle_partition_flag(struct fdisk_context *cxt, size_t i, unsign
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	if (i >= cxt->label->nparts_max)
 		return -EINVAL;
@@ -327,7 +344,7 @@ static void fetch_sun(struct fdisk_context *cxt,
 	assert(cxt);
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	sunlabel = self_disklabel(cxt);
 
@@ -386,15 +403,14 @@ static int sun_verify_disklabel(struct fdisk_context *cxt)
 #endif
     assert(cxt);
     assert(cxt->label);
-    assert(fdisk_is_disklabel(cxt, SUN));
+    assert(fdisk_is_label(cxt, SUN));
 
     fetch_sun(cxt, starts, lens, &start, &stop);
 
     for (k = 0; k < 7; k++) {
 	for (i = 0; i < SUN_MAXPARTITIONS; i++) {
-	    if (k && (lens[i] % (cxt->geom.heads * cxt->geom.sectors))) {
-	        fdisk_warnx(cxt, _("Partition %d doesn't end on cylinder boundary."), i+1);
-	    }
+	    if (k && (lens[i] % (cxt->geom.heads * cxt->geom.sectors)))
+	        fdisk_warnx(cxt, _("Partition %u doesn't end on cylinder boundary."), i+1);
 	    if (lens[i]) {
 	        for (j = 0; j < i; j++)
 	            if (lens[j]) {
@@ -413,8 +429,8 @@ static int sun_verify_disklabel(struct fdisk_context *cxt)
 	                        endo = starts[i]+lens[i];
 	                        if (starts[j]+lens[j] < endo)
 					endo = starts[j]+lens[j];
-	                        fdisk_warnx(cxt, _("Partition %d overlaps with others in "
-				       "sectors %d-%d."), i+1, starto, endo);
+	                        fdisk_warnx(cxt, _("Partition %u overlaps with others in "
+				       "sectors %u-%u."), i+1, starto, endo);
 	                    }
 	                }
 	            }
@@ -441,39 +457,61 @@ static int sun_verify_disklabel(struct fdisk_context *cxt)
     }
     stop = cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors;
     if (starts[array[0]])
-        fdisk_warnx(cxt, _("Unused gap - sectors 0-%d."), starts[array[0]]);
+        fdisk_warnx(cxt, _("Unused gap - sectors 0-%u."), starts[array[0]]);
     for (i = 0; i < 7 && array[i+1] != -1; i++) {
-        fdisk_warnx(cxt, _("Unused gap - sectors %d-%d."),
+        fdisk_warnx(cxt, _("Unused gap - sectors %u-%u."),
 	       (starts[array[i]] + lens[array[i]]),
 	       starts[array[i+1]]);
     }
     start = (starts[array[i]] + lens[array[i]]);
     if (start < stop)
-        fdisk_warnx(cxt, _("Unused gap - sectors %d-%d."), start, stop);
+        fdisk_warnx(cxt, _("Unused gap - sectors %u-%u."), start, stop);
 #endif
     return 0;
 }
 
+
+static int is_free_sector(struct fdisk_context *cxt,
+		fdisk_sector_t s, uint32_t starts[], uint32_t lens[])
+{
+	size_t i;
+
+	for (i = 0; i < cxt->label->nparts_max; i++) {
+		if (lens[i] && starts[i] <= s
+		    && starts[i] + lens[i] > s)
+			return 0;
+	}
+	return 1;
+}
+
 static int sun_add_partition(
 		struct fdisk_context *cxt,
-		size_t n,
-		struct fdisk_parttype *t)
+		struct fdisk_partition *pa,
+		size_t *partno)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
 	uint32_t starts[SUN_MAXPARTITIONS], lens[SUN_MAXPARTITIONS];
-	struct sun_partition *part = &sunlabel->partitions[n];
-	struct sun_info *info = &sunlabel->vtoc.infos[n];
+	struct sun_partition *part;
+	struct sun_info *info;
 	uint32_t start, stop, stop2;
-	int whole_disk = 0, sys = t ? t->type : SUN_TAG_LINUX_NATIVE;
-	struct fdisk_ask *ask;
+	int whole_disk = 0;
+	int sys = pa && pa->type ? pa->type->code : SUN_TAG_LINUX_NATIVE;
 	int rc;
+	size_t n;
 
 	char mesg[256];
 	size_t i;
 	unsigned int first, last;
 
+	rc = fdisk_partition_next_partno(pa, cxt, &n);
+	if (rc)
+		return rc;
+
+	part = &sunlabel->partitions[n];
+	info = &sunlabel->vtoc.infos[n];
+
 	if (part->num_sectors && be16_to_cpu(info->id) != SUN_TAG_UNASSIGNED) {
-		fdisk_info(cxt, _("Partition %zd is already defined.  Delete "
+		fdisk_info(cxt, _("Partition %zu is already defined.  Delete "
 			"it before re-adding it."), n + 1);
 		return -EINVAL;
 	}
@@ -489,131 +527,152 @@ static int sun_add_partition(
 			return -EINVAL;
 		}
 	}
-	snprintf(mesg, sizeof(mesg), _("First %s"),
-			fdisk_context_get_unit(cxt, SINGULAR));
-	for (;;) {
-		ask = fdisk_new_ask();
-		if (!ask)
-			return -ENOMEM;
 
-		fdisk_ask_set_query(ask, mesg);
-		fdisk_ask_set_type(ask, FDISK_ASKTYPE_NUMBER);
+	if (pa && pa->start_follow_default)
+		first = start;
+	else if (pa && fdisk_partition_has_start(pa)) {
+		first = pa->start;
 
-		if (whole_disk) {
-			fdisk_ask_number_set_low(ask,     0);	/* minimal */
-			fdisk_ask_number_set_default(ask, 0);	/* default */
-			fdisk_ask_number_set_high(ask,    0);	/* maximal */
-		} else {
-			fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, start));	/* minimal */
-			fdisk_ask_number_set_default(ask, fdisk_scround(cxt, start));	/* default */
-			fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop));	/* maximal */
-		}
-		rc = fdisk_do_ask(cxt, ask);
-		first = fdisk_ask_number_get_result(ask);
-		fdisk_free_ask(ask);
+		if (!whole_disk && !is_free_sector(cxt, first, starts, lens))
+			return -ERANGE;
+	} else {
+		struct fdisk_ask *ask;
 
-		if (rc)
-			return rc;
+		snprintf(mesg, sizeof(mesg), _("First %s"),
+				fdisk_get_unit(cxt, FDISK_SINGULAR));
+		for (;;) {
+			ask = fdisk_new_ask();
+			if (!ask)
+				return -ENOMEM;
 
-		if (fdisk_context_use_cylinders(cxt))
-			first *= fdisk_context_get_units_per_sector(cxt);
-		else {
-			/* Starting sector has to be properly aligned */
-			int cs = cxt->geom.heads * cxt->geom.sectors;
-			int x = first % cs;
+			fdisk_ask_set_query(ask, mesg);
+			fdisk_ask_set_type(ask, FDISK_ASKTYPE_NUMBER);
 
-			if (x) {
-				fdisk_info(cxt, _("Aligning the first sector from %u to %u "
-						  "to be on cylinder boundary."),
-						first, first + cs - x);
-				first += cs - x;
+			if (whole_disk) {
+				fdisk_ask_number_set_low(ask,     0);	/* minimal */
+				fdisk_ask_number_set_default(ask, 0);	/* default */
+				fdisk_ask_number_set_high(ask,    0);	/* maximal */
+			} else {
+				fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, start));	/* minimal */
+				fdisk_ask_number_set_default(ask, fdisk_scround(cxt, start));	/* default */
+				fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop));	/* maximal */
 			}
-		}
-		if (n == 2 && first != 0)
-			fdisk_warnx(cxt, _("It is highly recommended that the "
-				"third partition covers the whole disk "
-				"and is of type `Whole disk'"));
-		/* ewt asks to add: "don't start a partition at cyl 0"
-		   However, edmundo@rano.demon.co.uk writes:
-		   "In addition to having a Sun partition table, to be able to
-		   boot from the disc, the first partition, /dev/sdX1, must
-		   start at cylinder 0. This means that /dev/sdX1 contains
-		   the partition table and the boot block, as these are the
-		   first two sectors of the disc. Therefore you must be
-		   careful what you use /dev/sdX1 for. In particular, you must
-		   not use a partition starting at cylinder 0 for Linux swap,
-		   as that would overwrite the partition table and the boot
-		   block. You may, however, use such a partition for a UFS
-		   or EXT2 file system, as these file systems leave the first
-		   1024 bytes undisturbed. */
-		/* On the other hand, one should not use partitions
-		   starting at block 0 in an md, or the label will
-		   be trashed. */
-		for (i = 0; i < cxt->label->nparts_max; i++)
-			if (lens[i] && starts[i] <= first
-			            && starts[i] + lens[i] > first)
+			rc = fdisk_do_ask(cxt, ask);
+			first = fdisk_ask_number_get_result(ask);
+			fdisk_unref_ask(ask);
+			if (rc)
+				return rc;
+
+			if (fdisk_use_cylinders(cxt))
+				first *= fdisk_get_units_per_sector(cxt);
+
+			/* ewt asks to add: "don't start a partition at cyl 0"
+			   However, edmundo@rano.demon.co.uk writes:
+			   "In addition to having a Sun partition table, to be able to
+			   boot from the disc, the first partition, /dev/sdX1, must
+			   start at cylinder 0. This means that /dev/sdX1 contains
+			   the partition table and the boot block, as these are the
+			   first two sectors of the disc. Therefore you must be
+			   careful what you use /dev/sdX1 for. In particular, you must
+			   not use a partition starting at cylinder 0 for Linux swap,
+			   as that would overwrite the partition table and the boot
+			   block. You may, however, use such a partition for a UFS
+			   or EXT2 file system, as these file systems leave the first
+			   1024 bytes undisturbed. */
+			/* On the other hand, one should not use partitions
+			   starting at block 0 in an md, or the label will
+			   be trashed. */
+			if (!is_free_sector(cxt, first, starts,  lens) && !whole_disk) {
+				if (n == 2 && !first) {
+				    whole_disk = 1;
+				    break;
+				}
+				fdisk_warnx(cxt, _("Sector %d is already allocated"), first);
+			} else
 				break;
-		if (i < cxt->label->nparts_max && !whole_disk) {
-			if (n == 2 && !first) {
-			    whole_disk = 1;
-			    break;
-			}
-			fdisk_warnx(cxt, _("Sector %d is already allocated"), first);
-		} else
-			break;
+		}
 	}
+
+	if (n == 2 && first != 0)
+		fdisk_warnx(cxt, _("It is highly recommended that the "
+				   "third partition covers the whole disk "
+				   "and is of type `Whole disk'"));
+
+	if (!fdisk_use_cylinders(cxt)) {
+		/* Starting sector has to be properly aligned */
+		int cs = cxt->geom.heads * cxt->geom.sectors;
+		int x = first % cs;
+
+		if (x) {
+			fdisk_info(cxt, _("Aligning the first sector from %u to %u "
+					  "to be on cylinder boundary."),
+					first, first + cs - x);
+			first += cs - x;
+		}
+	}
+
 	stop = cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors;	/* ancient */
 	stop2 = stop;
 	for (i = 0; i < cxt->label->nparts_max; i++) {
 		if (starts[i] > first && starts[i] < stop)
 			stop = starts[i];
 	}
-	snprintf(mesg, sizeof(mesg),
-		 _("Last %s or +%s or +size{K,M,G,T,P}"),
-		 fdisk_context_get_unit(cxt, SINGULAR),
-		 fdisk_context_get_unit(cxt, PLURAL));
 
-	ask = fdisk_new_ask();
-	if (!ask)
-		return -ENOMEM;
+	/* last */
+	if (pa && pa->end_follow_default)
+		last = whole_disk || (n == 2 && !first) ? stop2 : stop;
+	else if (pa && fdisk_partition_has_size(pa)) {
+		last = first + pa->size - 1ULL;
 
-	fdisk_ask_set_query(ask, mesg);
-	fdisk_ask_set_type(ask, FDISK_ASKTYPE_OFFSET);
-
-	if (whole_disk) {
-		fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, stop2));	/* minimal */
-		fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop2));	/* default */
-		fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop2));	/* maximal */
-		fdisk_ask_number_set_base(ask,    0);
-	} else if (n == 2 && !first) {
-		fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, first));	/* minimal */
-		fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop2));	/* default */
-		fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop2));	/* maximal */
-		fdisk_ask_number_set_base(ask,	  fdisk_scround(cxt, first));
+		if (!whole_disk && last > stop)
+			return -ERANGE;
 	} else {
-		fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, first));	/* minimal */
-		fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop));	/* default */
-		fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop));	/* maximal */
-		fdisk_ask_number_set_base(ask,    fdisk_scround(cxt, first));
+		struct fdisk_ask *ask = fdisk_new_ask();
+
+		if (!ask)
+			return -ENOMEM;
+
+		snprintf(mesg, sizeof(mesg),
+			 _("Last %s or +%s or +size{K,M,G,T,P}"),
+			 fdisk_get_unit(cxt, FDISK_SINGULAR),
+			 fdisk_get_unit(cxt, FDISK_PLURAL));
+		fdisk_ask_set_query(ask, mesg);
+		fdisk_ask_set_type(ask, FDISK_ASKTYPE_OFFSET);
+
+		if (whole_disk) {
+			fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, stop2));	/* minimal */
+			fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop2));	/* default */
+			fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop2));	/* maximal */
+			fdisk_ask_number_set_base(ask,    0);
+		} else if (n == 2 && !first) {
+			fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, first));	/* minimal */
+			fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop2));	/* default */
+			fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop2));	/* maximal */
+			fdisk_ask_number_set_base(ask,	  fdisk_scround(cxt, first));
+		} else {
+			fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, first));	/* minimal */
+			fdisk_ask_number_set_default(ask, fdisk_scround(cxt, stop));	/* default */
+			fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop));	/* maximal */
+			fdisk_ask_number_set_base(ask,    fdisk_scround(cxt, first));
+		}
+
+		if (fdisk_use_cylinders(cxt))
+			fdisk_ask_number_set_unit(ask,
+				     cxt->sector_size *
+				     fdisk_get_units_per_sector(cxt));
+		else
+			fdisk_ask_number_set_unit(ask,	cxt->sector_size);
+
+		rc = fdisk_do_ask(cxt, ask);
+		last = fdisk_ask_number_get_result(ask);
+
+		fdisk_unref_ask(ask);
+		if (rc)
+			return rc;
+		if (fdisk_use_cylinders(cxt))
+			last *= fdisk_get_units_per_sector(cxt);
 	}
 
-	if (fdisk_context_use_cylinders(cxt))
-		fdisk_ask_number_set_unit(ask,
-			     cxt->sector_size *
-			     fdisk_context_get_units_per_sector(cxt));
-	else
-		fdisk_ask_number_set_unit(ask,	cxt->sector_size);
-
-	rc = fdisk_do_ask(cxt, ask);
-	last = fdisk_ask_number_get_result(ask);
-
-	fdisk_free_ask(ask);
-	if (rc)
-		return rc;
-
-	if (fdisk_context_use_cylinders(cxt))
-		last *= fdisk_context_get_units_per_sector(cxt);
-
 	if (n == 2 && !first) {
 		if (last >= stop2) {
 		    whole_disk = 1;
@@ -621,10 +680,10 @@ static int sun_add_partition(
 		} else if (last > stop) {
 		    fdisk_warnx(cxt,
    _("You haven't covered the whole disk with the 3rd partition, but your value\n"
-     "%d %s covers some other partition. Your entry has been changed\n"
-     "to %d %s"),
-			(int) fdisk_scround(cxt, last), fdisk_context_get_unit(cxt, SINGULAR),
-			(int) fdisk_scround(cxt, stop), fdisk_context_get_unit(cxt, SINGULAR));
+     "%lu %s covers some other partition. Your entry has been changed\n"
+     "to %lu %s"),
+			(unsigned long) fdisk_scround(cxt, last), fdisk_get_unit(cxt, FDISK_SINGULAR),
+			(unsigned long) fdisk_scround(cxt, stop), fdisk_get_unit(cxt, FDISK_SINGULAR));
 		    last = stop;
 		}
 	} else if (!whole_disk && last > stop)
@@ -633,8 +692,10 @@ static int sun_add_partition(
 	if (whole_disk)
 		sys = SUN_TAG_WHOLEDISK;
 
-	set_sun_partition(cxt, n, first, last, sys);
+	set_partition(cxt, n, first, last, sys);
 	cxt->label->nparts_cur = count_used_partitions(cxt);
+	if (partno)
+		*partno = n;
 	return 0;
 }
 
@@ -648,7 +709,7 @@ static int sun_delete_partition(struct fdisk_context *cxt,
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	sunlabel = self_disklabel(cxt);
 	part = &sunlabel->partitions[partnum];
@@ -674,18 +735,15 @@ static int sun_delete_partition(struct fdisk_context *cxt,
 static int sun_list_disklabel(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel;
-	struct tt *tb = NULL;
-	size_t i;
-	int rc;
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	sunlabel = self_disklabel(cxt);
 
-	if (fdisk_context_display_details(cxt)) {
-		fdisk_colon(cxt,
+	if (fdisk_is_details(cxt)) {
+		fdisk_info(cxt,
 		_("Label geometry: %d rpm, %d alternate and %d physical cylinders,\n"
 		  "                %d extra sects/cyl, interleave %d:1"),
 		       be16_to_cpu(sunlabel->rpm),
@@ -693,76 +751,83 @@ static int sun_list_disklabel(struct fdisk_context *cxt)
 		       be16_to_cpu(sunlabel->pcyl),
 		       be16_to_cpu(sunlabel->apc),
 		       be16_to_cpu(sunlabel->intrlv));
-		fdisk_colon(cxt, _("Label ID: %s"), sunlabel->label_id);
-		fdisk_colon(cxt, _("Volume ID: %s"),
+		fdisk_info(cxt, _("Label ID: %s"), sunlabel->label_id);
+		fdisk_info(cxt, _("Volume ID: %s"),
 		       *sunlabel->vtoc.volume_id ? sunlabel->vtoc.volume_id : _("<none>"));
 	}
 
-	tb = tt_new_table(TT_FL_FREEDATA);
-	if (!tb)
-		return -ENOMEM;
+	return 0;
+}
 
-	tt_define_column(tb, _("Device"), 0.2, 0);
-	tt_define_column(tb, _("Flag"),     2, TT_FL_RIGHT);
-	tt_define_column(tb, _("Start"),    9, TT_FL_RIGHT);
-	tt_define_column(tb, _("End"),      9, TT_FL_RIGHT);
-	/* TRANSLATORS: keep one blank space behind 'Blocks' */
-	tt_define_column(tb, _("Blocks "),  9, TT_FL_RIGHT);
-	tt_define_column(tb, _("Id"),       2, TT_FL_RIGHT);
-	tt_define_column(tb, _("System"), 0.2, TT_FL_TRUNC);
+static struct fdisk_parttype *sun_get_parttype(
+		struct fdisk_context *cxt,
+		size_t n)
+{
+	struct sun_disklabel *sunlabel = self_disklabel(cxt);
+	struct fdisk_parttype *t;
 
-	for (i = 0 ; i < cxt->label->nparts_max; i++) {
-		struct sun_partition *part = &sunlabel->partitions[i];
-		uint16_t flags = be16_to_cpu(sunlabel->vtoc.infos[i].flags);
-		uint32_t start, len;
-		struct fdisk_parttype *t;
-		struct tt_line *ln;
-		char *p;
+	if (n >= cxt->label->nparts_max)
+		return NULL;
 
-		if (!part->num_sectors)
-			continue;
-		ln = tt_add_line(tb, NULL);
-		if (!ln)
-			continue;
-
-		start = be32_to_cpu(part->start_cylinder)
-				* cxt->geom.heads
-				* cxt->geom.sectors;
-
-		len = be32_to_cpu(part->num_sectors);
-		t = fdisk_get_partition_type(cxt, i);
-
-		p = fdisk_partname(cxt->dev_path, i + 1);
-		if (p)
-			tt_line_set_data(ln, 0, p);	/* devname */
-		if ((flags & SUN_FLAG_UNMNT || flags & SUN_FLAG_RONLY)
-		    && asprintf(&p, "%c%c",
-				flags & SUN_FLAG_UNMNT ? 'u' : ' ',
-				flags & SUN_FLAG_RONLY ? 'r' : ' ') > 0)
-			tt_line_set_data(ln, 1, p);	/* flags */
-		if (asprintf(&p, "%ju", (uintmax_t) fdisk_scround(cxt, start)) > 0)
-			tt_line_set_data(ln, 2, p);	/* start */
-		if (asprintf(&p, "%ju",	(uintmax_t) fdisk_scround(cxt, start + len - 1)) > 0)
-			tt_line_set_data(ln, 3, p);	/* end */
-		if (asprintf(&p, "%lu%c",
-				(unsigned long) len / 2,
-				len & 1 ? '+' : ' ') > 0)
-			tt_line_set_data(ln, 4, p);	/* blocks + flag */
-		if (asprintf(&p, "%2x", t->type) > 0)
-			tt_line_set_data(ln, 5, p);	/* type ID */
-		if (t->name)
-			tt_line_set_data(ln, 6, strdup(t->name)); /* type Name */
-
-		fdisk_free_parttype(t);
-	}
-
-	rc = fdisk_print_table(cxt, tb);
-	tt_free_table(tb);
-
-	return rc;
+	t = fdisk_label_get_parttype_from_code(cxt->label,
+			be16_to_cpu(sunlabel->vtoc.infos[n].id));
+	return t ? : fdisk_new_unknown_parttype(be16_to_cpu(sunlabel->vtoc.infos[n].id), NULL);
 }
 
 
+static int sun_get_partition(struct fdisk_context *cxt, size_t n,
+			      struct fdisk_partition *pa)
+{
+	struct sun_disklabel *sunlabel;
+	struct sun_partition *part;
+	uint16_t flags;
+	uint32_t start, len;
+
+	assert(cxt);
+	assert(cxt->label);
+	assert(fdisk_is_label(cxt, SUN));
+
+	if (n >= cxt->label->nparts_max)
+		return -EINVAL;
+
+	sunlabel = self_disklabel(cxt);
+	part = &sunlabel->partitions[n];
+
+	pa->used = part->num_sectors ? 1 : 0;
+	if (!pa->used)
+		return 0;
+
+	flags = be16_to_cpu(sunlabel->vtoc.infos[n].flags);
+	start = be32_to_cpu(part->start_cylinder)
+			* cxt->geom.heads * cxt->geom.sectors;
+	len = be32_to_cpu(part->num_sectors);
+
+	pa->type = sun_get_parttype(cxt, n);
+	if (pa->type && pa->type->code == SUN_TAG_WHOLEDISK)
+		pa->wholedisk = 1;
+
+	if (flags & SUN_FLAG_UNMNT || flags & SUN_FLAG_RONLY) {
+		if (asprintf(&pa->attrs, "%c%c",
+				flags & SUN_FLAG_UNMNT ? 'u' : ' ',
+				flags & SUN_FLAG_RONLY ? 'r' : ' ') < 0)
+			return -ENOMEM;
+	}
+
+	pa->start = start;
+	pa->size = len;
+
+	return 0;
+}
+
+/**
+ * fdisk_sun_set_alt_cyl:
+ * @cxt: context
+ *
+ * Sets number of alternative cylinders. This function uses libfdisk Ask API
+ * for dialog with user.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_sun_set_alt_cyl(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
@@ -779,6 +844,15 @@ int fdisk_sun_set_alt_cyl(struct fdisk_context *cxt)
 	return 0;
 }
 
+/**
+ * fdisk_sun_set_xcyl:
+ * @cxt: context
+ *
+ * Sets number of extra sectors per cylinder. This function uses libfdisk Ask API
+ * for dialog with user.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_sun_set_xcyl(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
@@ -794,6 +868,15 @@ int fdisk_sun_set_xcyl(struct fdisk_context *cxt)
 	return 0;
 }
 
+/**
+ * fdisk_sun_set_ilfact:
+ * @cxt: context
+ *
+ * Sets interleave factor. This function uses libfdisk Ask API for dialog with
+ * user.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_sun_set_ilfact(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
@@ -809,6 +892,15 @@ int fdisk_sun_set_ilfact(struct fdisk_context *cxt)
 	return 0;
 }
 
+/**
+ * fdisk_sun_set_rspeed
+ * @cxt: context
+ *
+ * Sets rotation speed. This function uses libfdisk Ask API for dialog with
+ * user.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_sun_set_rspeed(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
@@ -824,6 +916,15 @@ int fdisk_sun_set_rspeed(struct fdisk_context *cxt)
 	return 0;
 }
 
+/**
+ * fdisk_sun_set_pcylcount
+ * @cxt: context
+ *
+ * Sets number of physical cylinders. This function uses libfdisk Ask API for
+ * dialog with user.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_sun_set_pcylcount(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel = self_disklabel(cxt);
@@ -848,7 +949,7 @@ static int sun_write_disklabel(struct fdisk_context *cxt)
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	sunlabel = self_disklabel(cxt);
 
@@ -856,9 +957,11 @@ static int sun_write_disklabel(struct fdisk_context *cxt)
 	sunlabel->nhead = cpu_to_be16(cxt->geom.heads);
 	sunlabel->nsect = cpu_to_be16(cxt->geom.sectors);
 
-	if (cxt->geom.cylinders != be16_to_cpu(sunlabel->ncyl))
-		sunlabel->ncyl = cpu_to_be16( cxt->geom.cylinders
-				      - be16_to_cpu(sunlabel->acyl) );
+	if (cxt->geom.cylinders != be16_to_cpu(sunlabel->ncyl)) {
+		int a = cpu_to_be16(cxt->geom.cylinders);
+		int b = be16_to_cpu(sunlabel->acyl);
+		sunlabel->ncyl = a - b;
+	}
 
 	ush = (unsigned short *) sunlabel;
 
@@ -873,30 +976,10 @@ static int sun_write_disklabel(struct fdisk_context *cxt)
 	return 0;
 }
 
-static struct fdisk_parttype *sun_get_parttype(
-		struct fdisk_context *cxt,
-		size_t n)
-{
-	struct sun_disklabel *sunlabel = self_disklabel(cxt);
-	struct fdisk_parttype *t;
-
-	assert(cxt);
-	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
-
-	if (n >= cxt->label->nparts_max)
-		return NULL;
-
-	t = fdisk_get_parttype_from_code(cxt, be16_to_cpu(sunlabel->vtoc.infos[n].id));
-	if (!t)
-		t = fdisk_new_unknown_parttype(be16_to_cpu(sunlabel->vtoc.infos[n].id), NULL);
-	return t;
-}
-
-static int sun_set_parttype(
+static int sun_set_partition(
 		struct fdisk_context *cxt,
 		size_t i,
-		struct fdisk_parttype *t)
+		struct fdisk_partition *pa)
 {
 	struct sun_disklabel *sunlabel;
 	struct sun_partition *part;
@@ -904,46 +987,63 @@ static int sun_set_parttype(
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
 	sunlabel = self_disklabel(cxt);
 
-	if (i >= cxt->label->nparts_max || !t || t->type > UINT16_MAX)
+	if (i >= cxt->label->nparts_max)
 		return -EINVAL;
 
-	if (i == 2 && t->type != SUN_TAG_WHOLEDISK)
-		fdisk_info(cxt, _("Consider leaving partition 3 as Whole disk (5),\n"
-		         "as SunOS/Solaris expects it and even Linux likes it.\n"));
+	if (pa->type) {
+		struct fdisk_parttype *t = pa->type;
 
-	part = &sunlabel->partitions[i];
-	info = &sunlabel->vtoc.infos[i];
+		if (t->code > UINT16_MAX)
+			return -EINVAL;
 
-	if (t->type == SUN_TAG_LINUX_SWAP && !part->start_cylinder) {
-	    int yes, rc;
-	    rc = fdisk_ask_yesno(cxt,
-	      _("It is highly recommended that the partition at offset 0\n"
-	      "is UFS, EXT2FS filesystem or SunOS swap. Putting Linux swap\n"
-	      "there may destroy your partition table and bootblock.\n"
-	      "Are you sure you want to tag the partition as Linux swap?"), &yes);
-	    if (rc)
-		    return rc;
-	    if (!yes)
-		    return 1;
+		if (i == 2 && t->code != SUN_TAG_WHOLEDISK)
+			fdisk_info(cxt, _("Consider leaving partition 3 as Whole disk (5),\n"
+			         "as SunOS/Solaris expects it and even Linux likes it.\n"));
+
+		part = &sunlabel->partitions[i];
+		info = &sunlabel->vtoc.infos[i];
+
+		if (cxt->script == NULL &&
+		    t->code == SUN_TAG_LINUX_SWAP && !part->start_cylinder) {
+			int yes, rc;
+
+			rc = fdisk_ask_yesno(cxt,
+			      _("It is highly recommended that the partition at offset 0\n"
+			      "is UFS, EXT2FS filesystem or SunOS swap. Putting Linux swap\n"
+			      "there may destroy your partition table and bootblock.\n"
+			      "Are you sure you want to tag the partition as Linux swap?"), &yes);
+			if (rc)
+				return rc;
+			if (!yes)
+				return 1;
+		}
+
+		switch (t->code) {
+		case SUN_TAG_SWAP:
+		case SUN_TAG_LINUX_SWAP:
+			/* swaps are not mountable by default */
+			info->flags |= cpu_to_be16(SUN_FLAG_UNMNT);
+			break;
+		default:
+			/* assume other types are mountable;
+			   user can change it anyway */
+			info->flags &= ~cpu_to_be16(SUN_FLAG_UNMNT);
+			break;
+		}
+		info->id = cpu_to_be16(t->code);
 	}
 
-	switch (t->type) {
-	case SUN_TAG_SWAP:
-	case SUN_TAG_LINUX_SWAP:
-		/* swaps are not mountable by default */
-		info->flags |= cpu_to_be16(SUN_FLAG_UNMNT);
-		break;
-	default:
-		/* assume other types are mountable;
-		   user can change it anyway */
-		info->flags &= ~cpu_to_be16(SUN_FLAG_UNMNT);
-		break;
-	}
-	info->id = cpu_to_be16(t->type);
+	if (fdisk_partition_has_start(pa))
+		sunlabel->partitions[i].start_cylinder =
+			cpu_to_be32(pa->start / (cxt->geom.heads * cxt->geom.sectors));
+	if (fdisk_partition_has_size(pa))
+		sunlabel->partitions[i].num_sectors = cpu_to_be32(pa->size);
+
+	fdisk_label_set_changed(cxt->label, 1);
 	return 0;
 }
 
@@ -954,29 +1054,35 @@ static int sun_reset_alignment(struct fdisk_context *cxt __attribute__((__unused
 }
 
 
-static int sun_get_partition_status(
+static int sun_partition_is_used(
 		struct fdisk_context *cxt,
-		size_t i,
-		int *status)
+		size_t i)
 {
 	struct sun_disklabel *sunlabel;
 
 	assert(cxt);
 	assert(cxt->label);
-	assert(fdisk_is_disklabel(cxt, SUN));
+	assert(fdisk_is_label(cxt, SUN));
 
-	if (!status || i >= cxt->label->nparts_max)
-		return -EINVAL;
+	if (i >= cxt->label->nparts_max)
+		return 0;
 
 	sunlabel = self_disklabel(cxt);
-	*status = FDISK_PARTSTAT_NONE;
-
-	if (sunlabel->partitions[i].num_sectors)
-		*status = FDISK_PARTSTAT_USED;
-
-	return 0;
+	return sunlabel->partitions[i].num_sectors ? 1 : 0;
 }
 
+static const struct fdisk_field sun_fields[] =
+{
+	{ FDISK_FIELD_DEVICE,	N_("Device"),	 10,	0 },
+	{ FDISK_FIELD_START,	N_("Start"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_END,	N_("End"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_SECTORS,	N_("Sectors"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_CYLINDERS,N_("Cylinders"),  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_SIZE,	N_("Size"),	  5,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_TYPEID,	N_("Id"),	  2,	FDISK_FIELDFL_NUMBER },
+	{ FDISK_FIELD_TYPE,	N_("Type"),	0.1,	0 },
+	{ FDISK_FIELD_ATTR,	N_("Flags"),	  0,	FDISK_FIELDFL_NUMBER }
+};
 
 const struct fdisk_label_operations sun_operations =
 {
@@ -985,12 +1091,13 @@ const struct fdisk_label_operations sun_operations =
 	.verify		= sun_verify_disklabel,
 	.create		= sun_create_disklabel,
 	.list		= sun_list_disklabel,
-	.part_add	= sun_add_partition,
-	.part_delete	= sun_delete_partition,
-	.part_get_type	= sun_get_parttype,
-	.part_set_type	= sun_set_parttype,
 
-	.part_get_status = sun_get_partition_status,
+	.get_part	= sun_get_partition,
+	.set_part	= sun_set_partition,
+	.add_part	= sun_add_partition,
+	.del_part	= sun_delete_partition,
+
+	.part_is_used	= sun_partition_is_used,
 	.part_toggle_flag = sun_toggle_partition_flag,
 
 	.reset_alignment = sun_reset_alignment,
@@ -1016,8 +1123,9 @@ struct fdisk_label *fdisk_new_sun_label(struct fdisk_context *cxt)
 	lb->id = FDISK_DISKLABEL_SUN;
 	lb->op = &sun_operations;
 	lb->parttypes = sun_parttypes;
-	lb->nparttypes = ARRAY_SIZE(sun_parttypes);
-
+	lb->nparttypes = ARRAY_SIZE(sun_parttypes) - 1;
+	lb->fields = sun_fields;
+	lb->nfields = ARRAY_SIZE(sun_fields);
 	lb->flags |= FDISK_LABEL_FL_REQUIRE_GEOMETRY;
 
 	return lb;
