@@ -82,13 +82,10 @@ static int is_local_in_file(const char *user, const char *filename)
 int is_local(const char *user)
 {
 	int rv;
-	if ((rv = is_local_in_file(user, _PATH_PASSWD)) < 0) {
-		perror(_PATH_PASSWD);
-		fprintf(stderr, _("cannot open %s"), _PATH_PASSWD);
-		exit(1);
-	} else {
-		return rv;
-	}
+
+	if ((rv = is_local_in_file(user, _PATH_PASSWD)) < 0)
+		err(EXIT_FAILURE, _("cannot open %s"), _PATH_PASSWD);
+	return rv;
 }
 
 #ifdef TEST_PROGRAM
