@@ -67,20 +67,19 @@ static int fstrim_filesystem(const char *path, struct fstrim_range *rangetpl,
 	/* kernel modifies the range */
 	memcpy(&range, rangetpl, sizeof(range));
 
-	if (stat(path, &sb) == -1) {
-		warn(_("stat failed %s"), path);
+	fd = open(path, O_RDONLY);
+	if (fd < 0) {
+		warn(_("cannot open %s"), path);
+		return -1;
+	}
+	if (fstat(fd, &sb) == -1) {
+		warn(_("stat of %s failed"), path);
 		return -1;
 	}
 	if (!S_ISDIR(sb.st_mode)) {
 		warnx(_("%s: not a directory"), path);
 		return -1;
 	}
-
-	fd = open(path, O_RDONLY);
-	if (fd < 0) {
-		warn(_("cannot open %s"), path);
-		return -1;
-	}
 	errno = 0;
 	if (ioctl(fd, FITRIM, &range)) {
 		int rc = errno == EOPNOTSUPP || errno == ENOTTY ? 1 : -1;
@@ -118,7 +117,7 @@ static int has_discard(const char *devname, struct sysfs_cxt *wholedisk)
 	 * This is tricky to read the info from sys/, because the queue
 	 * atrributes are provided for whole devices (disk) only. We're trying
 	 * to reuse the whole-disk sysfs context to optimize this stuff (as
-	 * system usualy have just one disk only).
+	 * system usually have just one disk only).
 	 */
 	if (sysfs_devno_to_wholedisk(dev, NULL, 0, &disk) || !disk)
 		return 1;
@@ -139,6 +138,15 @@ static int has_discard(const char *devname, struct sysfs_cxt *wholedisk)
 	return rc == 0 && dg > 0;
 }
 
+
+static int uniq_fs_target_cmp(
+		struct libmnt_table *tb __attribute__((__unused__)),
+		struct libmnt_fs *a,
+		struct libmnt_fs *b)
+{
+	return !mnt_fs_streq_target(a, mnt_fs_get_target(b));
+}
+
 /*
  * fstrim --all follows "mount -a" return codes:
  *
@@ -154,6 +162,8 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 	struct sysfs_cxt wholedisk = UL_SYSFSCXT_EMPTY;
 	int cnt = 0, cnt_err = 0;
 
+	mnt_init_debug(0);
+
 	itr = mnt_new_iter(MNT_ITER_BACKWARD);
 	if (!itr)
 		err(MOUNT_EX_FAIL, _("failed to initialize libmount iterator"));
@@ -162,6 +172,9 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 	if (!tab)
 		err(MOUNT_EX_FAIL, _("failed to parse %s"), _PATH_PROC_MOUNTINFO);
 
+	/* de-duplicate the table */
+	mnt_table_uniq_fs(tab, 0, uniq_fs_target_cmp);
+
 	while (mnt_table_next_fs(tab, itr, &fs) == 0) {
 		const char *src = mnt_fs_get_srcpath(fs),
 			   *tgt = mnt_fs_get_target(fs);
@@ -199,7 +212,8 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 	}
 
 	sysfs_deinit(&wholedisk);
-	mnt_free_table(tab);
+	mnt_unref_table(tab);
+	mnt_free_iter(itr);
 
 	if (cnt && cnt == cnt_err)
 		return MOUNT_EX_FAIL;		/* all failed */
@@ -214,6 +228,10 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
 	      _(" %s [options] <mount point>\n"), program_invocation_short_name);
+
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("Discard unused blocks on a mounted filesystem.\n"), out);
+
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -a, --all           trim all mounted filesystems that are supported\n"), out);
 	fputs(_(" -o, --offset <num>  the offset in bytes to start discarding from\n"), out);
@@ -230,7 +248,7 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 
 int main(int argc, char **argv)
 {
-	char *path;
+	char *path = NULL;
 	int c, rc, verbose = 0, all = 0;
 	struct fstrim_range range;
 
@@ -301,7 +319,7 @@ int main(int argc, char **argv)
 	else {
 		rc = fstrim_filesystem(path, &range, verbose);
 		if (rc == 1) {
-			warnx(_("%s: discard operation not supported."), path);
+			warnx(_("%s: the discard operation is not supported"), path);
 			rc = EXIT_FAILURE;
 		}
 	}
