@@ -1,4 +1,3 @@
-
 #include <inttypes.h>
 
 #include "c.h"
@@ -82,12 +81,15 @@ int ipc_shm_get_limits(struct ipc_limits *lim)
 		lim->shmmni = path_read_u64(_PATH_PROC_IPC_SHMMNI);
 
 	} else {
-		struct shminfo shminfo;
+		struct shminfo *shminfo;
+		struct shmid_ds shmbuf;
 
-		if (shmctl(0, IPC_INFO, (struct shmid_ds *) &shminfo) < 0)
+		if (shmctl(0, IPC_INFO, &shmbuf) < 0)
 			return 1;
-		lim->shmmni = shminfo.shmmni;
-		lim->shmall = shminfo.shmall;
+		shminfo = (struct shminfo *) &shmbuf;
+		lim->shmmni = shminfo->shmmni;
+		lim->shmall = shminfo->shmall;
+		lim->shmmax = shminfo->shmmax;
 	}
 
 	return 0;
@@ -98,7 +100,7 @@ int ipc_shm_get_info(int id, struct shm_data **shmds)
 	FILE *f;
 	int i = 0, maxid;
 	struct shm_data *p;
-	struct shm_info dummy;
+	struct shmid_ds dummy;
 
 	p = *shmds = xcalloc(1, sizeof(struct shm_data));
 	p->next = NULL;
@@ -154,7 +156,7 @@ int ipc_shm_get_info(int id, struct shm_data **shmds)
 
 	/* Fallback; /proc or /sys file(s) missing. */
 shm_fallback:
-	maxid = shmctl(0, SHM_INFO, (struct shmid_ds *) &dummy);
+	maxid = shmctl(0, SHM_INFO, &dummy);
 
 	for (int j = 0; j <= maxid; j++) {
 		int shmid;
@@ -385,10 +387,6 @@ int ipc_msg_get_info(int id, struct msg_data **msgds)
 		if (id > -1) {
 			/* ID specified */
 			if (id == p->msg_perm.id) {
-				/*
-				 * FIXME: q_qbytes are not in /proc
-				 *
-				 */
 				if (msgctl(id, IPC_STAT, &msgseg) != -1)
 					p->q_qbytes = msgseg.msg_qbytes;
 				i = 1;
@@ -488,10 +486,10 @@ void ipc_print_perms(FILE *f, struct ipc_stat *is)
 		fprintf(f, " %-10u\n", is->gid);
 }
 
-void ipc_print_size(int unit, char *msg, size_t size, const char *end,
+void ipc_print_size(int unit, char *msg, uint64_t size, const char *end,
 		    int width)
 {
-	char format[16];
+	char format[32];
 
 	if (!msg)
 		/* NULL */ ;
@@ -507,11 +505,11 @@ void ipc_print_size(int unit, char *msg, size_t size, const char *end,
 	switch (unit) {
 	case IPC_UNIT_DEFAULT:
 	case IPC_UNIT_BYTES:
-		sprintf(format, "%%%dzu", width);
+		sprintf(format, "%%%dju", width);
 		printf(format, size);
 		break;
 	case IPC_UNIT_KB:
-		sprintf(format, "%%%dzu", width);
+		sprintf(format, "%%%dju", width);
 		printf(format, size / 1024);
 		break;
 	case IPC_UNIT_HUMAN:
