@@ -317,6 +317,10 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 {
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] <file>...\n"), program_invocation_short_name);
+
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("A file perusal filter for CRT viewing.\n"), out);
+
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -d          display help instead of ringing bell\n"), out);
 	fputs(_(" -f          count logical rather than screen lines\n"), out);
@@ -591,7 +595,7 @@ FILE *checkf(register char *fs, int *clearfirst)
 		fflush(stdout);
 		if (clreol)
 			cleareol();
-		perror(fs);
+		warn(_("stat of %s failed"), fs);
 		return ((FILE *)NULL);
 	}
 	if ((stbuf.st_mode & S_IFMT) == S_IFDIR) {
@@ -600,11 +604,13 @@ FILE *checkf(register char *fs, int *clearfirst)
 	}
 	if ((f = Fopen(fs, "r")) == NULL) {
 		fflush(stdout);
-		perror(fs);
+		warn(_("cannot open %s"), fs);
 		return ((FILE *)NULL);
 	}
-	if (magic(f, fs))
+	if (magic(f, fs)) {
+		fclose(f);
 		return ((FILE *)NULL);
+	}
 	fcntl(fileno(f), F_SETFD, FD_CLOEXEC);
 	c = Getc(f);
 	*clearfirst = (c == '\f');
@@ -637,7 +643,6 @@ static int magic(FILE *f, char *fs)
 		case 0x457f:	/* simple ELF detection */
 			printf(_("\n******** %s: Not a text file ********\n\n"),
 			       fs);
-			fclose(f);
 			return 1;
 		}
 	}
@@ -1090,6 +1095,16 @@ void clreos(void)
 	my_putstring(EodClr);
 }
 
+
+static UL_ASAN_BLACKLIST size_t xmbrtowc(wchar_t *wc, const char *s, size_t n,
+				  mbstate_t *mbstate)
+{
+	const size_t mblength = mbrtowc(wc, s, n, mbstate);
+	if (mblength == (size_t)-2 || mblength == (size_t)-1)
+		return 1;
+	return mblength;
+}
+
 /* Print a buffer of n characters */
 void prbuf(register char *s, register int n)
 {
@@ -1129,10 +1144,7 @@ void prbuf(register char *s, register int n)
 				memset(&mbstate, '\0', sizeof(mbstate_t));
 				s--;
 				n++;
-				mblength = mbrtowc(&wc, s, n, &mbstate);
-				if (mblength == (size_t)-2
-				    || mblength == (size_t)-1)
-					mblength = 1;
+				mblength = xmbrtowc(&wc, s, n, &mbstate);
 				while (mblength--)
 					putchar(*s++);
 				n += mblength;
@@ -1231,10 +1243,9 @@ int command(char *filename, register FILE *f)
 				putchar('\n');
 				if (clreol)
 					cleareol();
-				if (nlines != 1)
-					printf(_("...back %d pages"), nlines);
-				else
-					putsout(_("...back 1 page"));
+				printf(P_("...back %d page",
+					"...back %d pages", nlines),
+					nlines);
 				if (clreol)
 					cleareol();
 				putchar('\n');
@@ -1280,10 +1291,9 @@ int command(char *filename, register FILE *f)
 			putchar('\n');
 			if (clreol)
 				cleareol();
-			if (nlines == 1)
-				putsout(_("...skipping one line"));
-			else
-				printf(_("...skipping %d lines"), nlines);
+			printf(P_("...skipping %d line",
+				"...skipping %d lines", nlines),
+				nlines);
 
 			if (clreol)
 				cleareol();
@@ -1595,6 +1605,8 @@ void search(char buf[], FILE *file, register int n)
 	context.line = saveln = Currline;
 	context.chrctr = startline;
 	lncount = 0;
+	if (!buf)
+		goto notfound;
 	if ((rc = regcomp(&re, buf, REG_NOSUB)) != 0) {
 		char s[REGERR_BUF];
 		regerror(rc, &re, s, sizeof s);
@@ -1651,6 +1663,7 @@ void search(char buf[], FILE *file, register int n)
 		}
 		free(previousre);
 		previousre = NULL;
+notfound:
 		more_error(_("Pattern not found"));
 	}
 }
@@ -1782,10 +1795,8 @@ void initterm(void)
 			int tgrp;
 			/* Wait until we're in the foreground before we
 			 * save the terminal modes. */
-			if ((tgrp = tcgetpgrp(fileno(stdout))) < 0) {
-				perror("tcgetpgrp");
-				exit(EXIT_FAILURE);
-			}
+			if ((tgrp = tcgetpgrp(fileno(stdout))) < 0)
+				err(EXIT_FAILURE, "tcgetpgrp");
 			if (tgrp != getpgrp(0)) {
 				kill(0, SIGTTOU);
 				goto retry;
